<?php

namespace App\Controllers;

use App\Models\AdminModel;
use App\Models\AdminPermissionModel;
use App\Models\AdminRoleModel;
use App\Models\AdminModuleModel;
use App\Models\AdminRoleModuleModel;

class AdminRole extends BaseController
{
	public function getAll()
	{
		if($this->request->isAjax()){
			return $this->data();
		}

		$adminRoleModel = new AdminRoleModel();
		$adminRoleData = $adminRoleModel->getAll();

		$data = [
			'title' => lang('Menu.user.role.main'),
			'menu' => 'admin-role',
		];

		return view('admin-role/index', $data);
	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}

		$adminRoleModel = new AdminRoleModel();
		$adminRoleData = $adminRoleModel->getAll();

		$data = [];

		if (@$adminRoleData) {

			foreach ($adminRoleData as $index => $val ) {
        		
	            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/admin/role/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

	            $btn_edit = '&nbsp;<a href="'.base_url().'/admin/role/edit/'.$val->id.'" class="btn btn-icon btn-success" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</i></a>';

	            $value['no'] = $index+1;
	            $value['name'] = $val->name;
	            $value['description'] = $val->description;
	            $value['insert_at'] = $val->insert_at;
	            $value['action'] = $btn_edit.$btn_delete;

	            array_push($data,$value);
	        } 	
		}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }


	public function addAdminRole()
	{
		if (!$this->request->isAjax()) {
			$adminModuleModel = new AdminModuleModel();
			$adminModuleData = $adminModuleModel->getAll();

			$data = [
				'data_module' => $adminModuleData,
				'title' => lang('Menu.user.role.main'),
				'menu' => 'admin-role',
			];

			return view('admin-role/create', $data);
		}

		if (!$this->validate([
             'name'          => ['label' => 'Nama', 'rules' => 'required|min_length[3]'],
            'description'   => ['label' => 'Deskripsi', 'rules' => 'required|min_length[3]'],
            'module'   => ['label' => 'Modul', 'rules' => 'required'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		 $data = [
            'ar_name'               => $this->request->getPost('name'),
            'ar_slug'               => url_title($this->request->getPost('name'),"-",TRUE),
            'ar_description'        => $this->request->getPost('description'),
            'ar_insert_admin_id'    => $this->session->get('id'),
            'ar_active'             => 1
        ];

		$adminRoleModel = new AdminRoleModel();
		$adminRoleID = $adminRoleModel->addAdminRole($data);

		$moduleID = $this->request->getPost('module');

		$adminRoleModuleModel = new AdminRoleModuleModel();

		$adminRoleModuleData = $adminRoleModuleModel->addAdminRoleModuleBulk($adminRoleID, $moduleID,$this->session->get('id'));

		if ($adminRoleID && $adminRoleModuleData) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect'=> 'admin/role'
			];
			return json_encode($data);
		}
	}

	public function updateAdminRole($id)
	{
		if (!$this->request->isAjax()) {
			$adminRoleModel = new AdminRoleModel();
			$adminRoleData = $adminRoleModel->getByID($id);

			$adminRoleModuleModel = new AdminRoleModuleModel();
			$adminRoleModuleData = $adminRoleModuleModel->getByRoleID($id);

			$adminModuleModel = new AdminModuleModel();
			$adminModuleData = $adminModuleModel->getAll();

			$arrayResultRoleModule = [];

			if(@$adminRoleModuleData){
				foreach ($adminRoleModuleData as $key => $value) {
					$arrayResultRoleModule[] = $value->module_id;
				}
			}

			$data = [
				'data' => (@$adminRoleData )?$adminRoleData : [],
				'data_role_module' => $arrayResultRoleModule,
				'data_module' => (@$adminModuleData)? $adminModuleData : [],
				'title' => lang('Menu.user.role.main'),
				'menu' => 'admin-role',
			];

			return view('admin-role/edit', $data);
		}

		if (!$this->validate([
             'name'          => ['label' => 'Name', 'rules' => 'required|min_length[3]'],
            'description'   => ['label' => 'Description', 'rules' => 'required|min_length[3]'],
            'module'   => ['label' => 'Module', 'rules' => 'required'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		 $data = [
            'ar_name'               => $this->request->getPost('name'),
            'ar_slug'               => url_title($this->request->getPost('name'),"-",TRUE),
            'ar_description'        => $this->request->getPost('description'),
            'ar_update_admin_id'    => $this->session->get('id'),
            'ar_active'             => 1
        ];

		$adminRoleModel = new AdminRoleModel();
		$adminRoleData = $adminRoleModel->updateAdminRole($data, $id);

		$moduleID = $this->request->getPost('module');

		$adminRoleModuleModel = new AdminRoleModuleModel();

		$adminRoleModuleData = $adminRoleModuleModel->updateAdminRoleModuleBulk($id, $moduleID,$this->session->get('id'));

		if ($adminRoleData && $adminRoleModuleData) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect'=> 'admin/role'
			];
			return json_encode($data);
		}
	}

	public function deleteAdminRole($roleID)
	{

		$adminRoleModel = new AdminRoleModel();
        $updateResult = $adminRoleModel->deleteAdminRole($roleID, $this->session->get('id'));

		if ($updateResult) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];
			return json_encode($data);
		}
	}
}
