<?php
namespace App\Controllers;

use Ramsey\Uuid\Uuid;
use App\Libraries\S3Library;
use App\Libraries\Barcode;

use App\Models\BatchItemOutputModel;
use App\Models\BatchItemModel;
use App\Models\BatchModel;


class BatchItemOutput extends BaseController
{
	public function index()
	{
		$act = $this->request->getVar('act');

		if($act == 'export'){
			return $this->export();
		}

		if($this->request->isAjax()){
			return $this->data();
		}

		$data = [
			'title' => lang('Menu.order.batch-item-output.main'),
			'menu' => 'batch-item-output',
		];

		return view('batch-item-output/index', $data);
	}

	public function store(){

		$rules = [
            'item_quantity' => ['label' => 'Jumlah', 'rules' => 'required|max_length[11]'],
            'item_weight' => ['label' => 'Weight', 'rules' => 'required|max_length[65]'],
        ];

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$isMelt = $this->request->getVar('is_melt');
		$biID = $this->request->getVar('bi_id');
		$bitID = $this->request->getVar('bit_id');

		if ($isMelt == '') {
			$isMelt = 0;
		}else{
			$isMelt = 1;
		}

		$batchModel = new BatchModel();
		$batchItemOutputModel = new BatchItemOutputModel();

		$batchItemModel = new BatchItemModel();

		$biData = $batchItemModel->getByID($biID);

		if ($biData == false) {
			$msg = [
                'error' => lang('Validation.data_not_found'),
            ];
            return json_encode($msg);
		}
		
		$data = [
			'bio_bi_id'=> $biID,
			'bio_oi_id'=> @$biData[0]->order_item_id,
			'bio_item_id'=> @$biData[0]->item_id,
			'bio_is_melt'=> $isMelt,
			'bio_bit_id' => $bitID,
			'bio_note'=> $this->bioNote(date('Y-m-d')),
			'bio_item_sku' => @$biData[0]->sku,
			'bio_item_name' => @$biData[0]->name,
			'bio_item_quantity' => $this->request->getPost('item_quantity', FILTER_SANITIZE_STRING),
			'bio_item_weight' => $this->request->getPost('item_weight', FILTER_SANITIZE_STRING),
			'bio_insert_at' => getCurrentTime(),
			'bio_insert_admin_id' => $this->session->get('id')
		];

		$result = $batchItemOutputModel->addBatchItemOutput($data);

		if ($result) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'modal_close' => '#kt_modal_melt'
			];

			return json_encode($data);
		}
	}


	public function data(){
		// if (!$this->request->isAjax()) {
		// 	return false;
		// }

		$orderItem = $this->request->getVar('order_item');
		$batchItem = $this->request->getVar('batch_item');		

		$batchItemOutputModel = new BatchItemOutputModel();
		if ($orderItem != '') {
			$batchItemOutputData = $batchItemOutputModel->getAll(['where' => ' AND bio_oi_id='.$orderItem.'  ',  'orderby' => ' bio_id ASC ']);
		}else if($batchItem != ''){
			$batchItemOutputData = $batchItemOutputModel->getAll(['where' => ' AND bio_bi_id='.$orderItem.'  ',  'orderby' => ' bio_id ASC ']);
		}

		$data = [];

		if (@$batchItemOutputData) {
        	foreach ($batchItemOutputData as $index => $val ) {

            $value['no'] = $index+1;
            $value['item_sku'] = $val->item_sku;
            $value['item_name'] = $val->item_name;
            $value['is_melt'] = ($val->is_melt == 1)? 'Lebur' : '';
            $value['item_quantity'] = $val->item_quantity;
            $value['item_weight'] = $val->item_weight;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
	}

	public function export(){
		$orderItem = $this->request->getVar('order_item');
		$batchItem = $this->request->getVar('batch_item');		

		$batchItemOutputModel = new BatchItemOutputModel();
		if ($orderItem != '') {
			$batchItemOutputData = $batchItemOutputModel->getAll(['where' => ' AND bio_oi_id='.$orderItem.'  ',  'orderby' => ' bio_id ASC ']);
		}else if($batchItem != ''){
			$batchItemOutputData = $batchItemOutputModel->getAll(['where' => ' AND bio_bi_id='.$orderItem.'  ',  'orderby' => ' bio_id ASC ']);
		}


    	$data = [
			'title' => lang('Menu.order.batch-item-output.main'),
			'menu' => 'batch-item-output',
			'data' => $batchItemOutputData
		];

		return view('batch-item-output/export', $data);
	}


	public function bioNote($data){

		$batchItemOutputModel = new BatchItemOutputModel();

		$bitNota = $batchItemOutputModel->setNote();


		if ($bitNota) {
			$number = $bitNota[0]->total;
	        $number = ((int)$number) + 1;
	        $code = str_pad($number, 5, "0", STR_PAD_LEFT);
		}else{
			$code = '00001';
		}

		$dateEx = explode('-', date('Y-m-d'));

		return 'ONBP/'.$dateEx[2].$dateEx[1].substr($dateEx[0], -2) .'/'.$code;
	}


}