<?php
namespace App\Controllers;

use App\Models\OrderModel;
use App\Models\OrderItemModel;
use App\Models\CustomerModel;
use App\Models\WorkflowModel;
use App\Models\SizeModel;
use App\Models\GoldContentModel;
use App\Models\CategoryModel;
use App\Models\ItemModel;
use Ramsey\Uuid\Uuid;
use App\Libraries\S3Library;

class OrderItem extends BaseController
{
	public function getAll()
	{
        if($this->request->isAjax()){
            return $this->data();
        }
		$data = [
			'title' => lang('Menu.order.order-item.main'),
			'menu' => 'order-item',
		];

		return view('order-item/index', $data);
	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}
		
		$orderItemModel = new OrderItemModel();

		$orderItemData = $orderItemModel->getAll();

		$data = [];

		if (@$orderItemData) {
        	foreach ($orderItemData as $index => $val ) {
        		
	            $btn_save = '<a href="javascript:;" class="btn btn-icon btn-primary update-row-data button'.$val->id.'" '.lang('Action.tooltip.save').'>'.lang('Action.icon.lg.save').'</a><div class="status_save status_save'.$val->id.'"></div>';

	            $value['no'] = ($index+1);
	            $value['start_date'] = '<input value="'.$val->start_date.'" name="start_date" class="form-control" type="date">
	            	<input value="'.$val->id.'" name="order_item_id" class="form-control" type="hidden">';
	            $value['note'] = $val->order_note;
	            $value['name'] = $val->name;
	            $value['category'] = $val->category_name;
	            $value['gold_content'] = $val->gc_name;
	            $value['quantity'] = $val->quantity;
	            $value['completed_quantity'] = $val->completed_quantity;
	            $value['status'] = $val->order_item_status;
	            $value['weight'] = $val->weight;
	            $value['size'] = $val->size_name;
	            $value['use_wire'] = $val->use_wire;
	            $value['stone_type'] = $val->stone_type;
	            $value['print_no'] = $val->print_no;
	            $value['action'] = $btn_save;

	            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }


	public function updateOrderItem()
	{

		$rules = [
			'order_item_id'             => ['label' => 'Order Barang', 'rules' => 'required|max_length[11]'],
            'start_date'             => ['label' => 'Tanggal Mulai', 'rules' => 'required|max_length[255]'],
        ];

        if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$orderItemModel = new OrderItemModel();

		$data = [
			'oi_start_date' 		    => $this->request->getPost('start_date'),
            'oi_update_admin_id' => $this->session->get('id'),
            'oi_status' => 'Proses',
        ];

		$orderItemData = $orderItemModel->updateOrderItem($data, $this->request->getPost('order_item_id')); 

		if ($orderItemData) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'order'
			];

			return json_encode($data);
		}

	}

	public function deleteOrder($id)
	{

		$orderModel = new OrderModel();

		$orderData = $orderModel->deleteOrder($id, $this->session->get('id')); 

		if ($orderData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}

}