<?php

namespace App\Controllers;

use App\Models\TreeModel;

class Tree extends BaseController
{
	public function getAll()
	{
		if($this->request->isAjax()){
			return $this->data();
		}
		
		$data = [
			'title' => lang('Menu.master.tree.main'),
			'menu' => 'tree',
		];

		return view('tree/index', $data);
	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}
		
		$treeModel = new TreeModel();

		$treeData = $treeModel->getAll(); 

		$data = [];

		if (@$treeData) {
        	foreach ($treeData as $index => $val ) {
        		
            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/tree/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

            $btn_edit = '&nbsp;<a href="'.base_url().'/tree/edit/'.$val->id.'" class="btn btn-success btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

            $value['no'] = $index+1;
            $value['code'] = $val->code;
            $value['name'] = $val->name;
            $value['weight'] = $val->weight;
            $value['insert_at'] = $val->insert_at;
            $value['action'] = $btn_edit.$btn_delete;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

	public function addTree()
	{
		if (!$this->request->isAjax()) {
			$data = [
				'title' => lang('Menu.master.tree.main'),
				'menu' => 'tree',
			];

			return view('tree/create', $data);
		}

		if (!$this->validate([
            'name'             => ['label' => 'Nama Pohon', 'rules' => 'required|max_length[255]'],
            'weight'             => ['label' => 'Berat', 'rules' => 'required|numeric|max_length[255]|' ],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$data = [
			'tree_code'		     => url_title($this->request->getPost('name'), "-", TRUE),
			'tree_name' 		     => $this->request->getPost('name'),
			'tree_weight' 		     => $this->request->getPost('weight'),
            'tree_insert_admin_id' => $this->session->get('id'),
            'tree_active'          => 1
        ];

		$treeModel = new TreeModel();

        $treeData = $treeModel->getByCode(url_title($this->request->getPost('name'), "-", TRUE));
        if (count($treeData) > 0) {
				$msg = [
	                'error' => lang('Validation.name_already_exist'),
	            ];
	            return json_encode($msg);
        }


		$treeData = $treeModel->addTree($data); 
		

		if ($treeData) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect' => 'tree'
			];

			return json_encode($data);
		}

	}

	public function updateTree($id)
	{
		$treeModel = new TreeModel();

		$treeData = $treeModel->getByID($id);
		if (!$this->request->isAjax()) {

			$data = [
				'data' => (isset($treeData))? $treeData : [],
				'title' => lang('Menu.master.tree.main'),
				'menu' => 'tree',
			];

			return view('tree/edit',  $data);
		}

		$data = [
			'tree_code'		     => url_title($this->request->getPost('name'), "-", TRUE),
			'tree_name' 		     => $this->request->getPost('name'),
			'tree_weight' 		     => $this->request->getPost('weight'),
            'tree_update_admin_id' => $this->session->get('id'),
            'tree_active'          => 1
        ];

		$rules = [
            'weight' => ['label' => 'Berat', 'rules' => 'required|numeric|max_length[255]|' ],
            'name' => ['label' => 'Nama Pohon', 'rules' => 'required|max_length[255]']
        ];

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

        if ($treeData->name != $data['tree_name']) {
        	$treeDataByCode = $treeModel->getByCode(url_title($this->request->getPost('name'), "-", TRUE));
	        if (count($treeDataByCode) > 0) {
					$msg = [
		                'error' => lang('Validation.name_already_exist'),
		            ];
		            return json_encode($msg);
	        }
        }

		$data2 = [
			'th_tree_id'		     => $id,
			'th_weight_from' 		 => $treeData->weight,
			'th_weight_to' 		     => $data['tree_weight'],
            'th_update_admin_id' => $this->session->get('id'),
        ];

		$treeID = $treeModel->updateTree($data, $id);
		$treeHistoryID = $treeModel->addTreeHistory($data2); 

		if ($treeID) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'tree'
			];

			return json_encode($data);
		}
	}

	public function deleteTree($id)
	{
		
		$treeModel = new TreeModel();

		$treeData = $treeModel->deleteTree($id, $this->session->get('id')); 

		if ($treeData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}

}