<?php namespace App\Models;

class AdminModel extends BaseModel
{
    public function __construct(){
        parent::__construct();
    }

    public function getAll($param=array()){
        if (!empty($param['select']))
            $query = 'SELECT '.$param["select"];
        else        
            $query = 'SELECT admin_id as "id", admin_fullname as "fullname", admin_email as "email", admin_phone as "phone", admin_allowed_login as "allow_login", admin_last_login as "last_login", admin_insert_at as "insert_at" ';

        $query .= ' FROM admins
            WHERE 1 ';

        if(!empty($param['where']))
            $query .= ' '.$param['where'];
        else
            $query .= ' AND admin_active=1 AND admin_allowed_login=1 ';

        if(!empty($param['groupby']))
            $query .= ' GROUP BY '.$param['groupby'];

        else
            $query .= ' GROUP BY admin_id ';


        if(!empty($param['orderBy']))
            $query .= ' ORDER BY '.$param['orderby'];
        

        if(!empty($param['limit']))
            $query .= ' LIMIT '.$param['limit'];
        else
            $query .= '';

        if(!empty($param['resultType'])) 
            return $this->db->query($query);
        else
            return $this->db->query($query)->getResult();
    }

    public function getByID($adminID){
        $admins = $this->db->table('admins');
        $admins->select('admin_id as "id", admin_fullname as "fullname", admin_email as "email", admin_phone as "phone", admin_allowed_login as "allow_login", admin_last_login as "last_login", admin_insert_at as "insert_at" ');
        $admins->where('admin_id', $adminID);
        $admins->where('admin_allowed_login', 1);
        $admins->where('admin_active', 1);

        $result = $admins->get()->getRow();

        return $result;
    }

    public function getPasswordByID($adminID){
        $admins = $this->db->table('admins');
        $admins->select(' admin_password as "password" ');
        $admins->where('admin_id', $adminID);
        $admins->where('admin_allowed_login', 1);
        $admins->where('admin_active', 1);

        $result = $admins->get()->getRow();

        return $result;
    }

    public function getByEmail($adminEmail){
        $param = array(
            'where' => ' AND admin_email="'.$adminEmail.'" AND admin_active="1" AND admin_allowed_login=1 ',
        );
        $result = $this->getAll($param);

        return $result;
    }

    public function getByForgotPasswordToken($forgotPasswordToken){
        $admins = $this->db->table('admins');
        $admins->select('admin_id as "id", admin_email as "email", admin_forgot_password_token as "forgot_password_token", admin_forgot_password_token_expired_at as "forgot_password_expired_at" ');
        $admins->where('admin_forgot_password_token', $forgotPasswordToken);
        $admins->where('admin_allowed_login', 1);
        $admins->where('admin_active', 1);

        $result = $admins->get()->getRow();

        return $result;
    }

    public function checkLogin($adminEmail, $adminPassword){
        $admins = $this->db->table('admins');
        $admins->select('admin_id as "id", admin_fullname as "fullname", admin_email as "email", admin_password, admin_phone as "phone", admin_allowed_login as "allow_login", admin_last_login as "last_login", admin_insert_at as "insert_at" ');
        $admins->where('admin_email', $adminEmail);
        $admins->where('admin_allowed_login', 1);
        $admins->where('admin_active', 1);

        $query = $admins->get();
        $adminData = $query->getRow();
        $verifiedAdmin = FALSE; 
        if($adminData && isset($adminData->admin_password)){
            $verifiedAdmin = password_verify($adminPassword, $adminData->admin_password);
            $refreshToken = hashingPassword($adminData->fullname.date('YmdHis').$adminData->email);
            $refreshTokenExpiredAt = time()+getenv('ACCESS_TOKEN_EXPIRED');
        }

        $result = FALSE;
        if($verifiedAdmin){
            unset($adminData->admin_password);
            $data = [
                'admin_last_login' => getCurrentTime(),
                'admin_refresh_token' => $refreshToken,
                'admin_refresh_token_expired_at' => $refreshTokenExpiredAt
            ];
            $admins = $this->db->table('admins')->where('admin_id', $adminData->id)->update($data);            
            $adminData->refresh_token = $refreshToken;
            $adminData->refresh_token_expired_at = $refreshTokenExpiredAt;
            $result = $adminData;
        }

        return $result;
    }


    public function refreshToken($adminID, $refreshToken){
        $admins = $this->db->table('admins');
        $admins->select('admin_id as "id", admin_fullname as "fullname", admin_email as "email", admin_password, admin_phone as "phone", admin_allowed_login as "allow_login", admin_last_login as "last_login", admin_insert_at as "insert_at" ');
        $admins->where('admin_id', $adminID);
        $admins->where('admin_refresh_token', $refreshToken);
        $admins->where('admin_refresh_token_expired_at >=', time());
        $admins->where('admin_allowed_login', 1);
        $admins->where('admin_active', 1);

        $query = $admins->get();
        $adminData = $query->getRow();
        $result = FALSE; 
        if($adminData && isset($adminData->admin_password)){
            $refreshToken = hashingPassword($adminData->fullname.date('YmdHis').$adminData->email);
            $refreshTokenExpiredAt = time()+getenv('ACCESS_TOKEN_EXPIRED');

            unset($adminData->admin_password);
            $data = [
                'admin_last_login' => getCurrentTime(),
                'admin_refresh_token' => $refreshToken,
                'admin_refresh_token_expired_at' => $refreshTokenExpiredAt
            ];
            $admins = $this->db->table('admins')->where('admin_id', $adminData->id)->update($data);            
            $adminData->refresh_token = $refreshToken;
            $adminData->refresh_token_expired_at = $refreshTokenExpiredAt;
            $result = $adminData;
        }

        return $result;
    }

    public function forgotPasswordToken($adminEmail){
        $param = array(
            'where' => ' AND admin_email="'.$adminEmail.'" AND admin_active="1" AND admin_allowed_login=1 ',
        );
        $adminData = $this->getAll($param);
        $forgotToken = $token = bin2hex(random_bytes(24));
        $forgotTokenExpiredAt = time()+getenv('FORGOT_PASSWORD_TOKEN_EXPIRED');
        $data = array(
            'admin_forgot_password_token'               => $forgotToken,
            'admin_forgot_password_token_expired_at'    => $forgotTokenExpiredAt
        );
        $adminData = $this->db->table('admins')->where('admin_id', $adminData[0]->id)->update($data);     
        return $adminData;
    }

    public function updatePassword($adminID, $password ){
        $data = [
            'admin_password'                            => hashingPassword($password),
            'admin_forgot_password_token'               => null,
            'admin_forgot_password_token_expired_at'    => null
        ];

        $this->db->table('admins')->where('admin_id',$adminID)->update($data);
        $param = array(
            'where' => ' AND admin_id="'.$adminID.'" ',
        );
        $result = $this->getAll($param);
        if ($result)
            $result = $result[0];

        return $result;
        
    }

    public function addAdmin($data){
        $admins = $this->db->table('admins');
        $admins->insert($data);
        $result = $this->db->insertID();

        return $result;
    }

    public function updateAdmin($data, $adminID){

        $this->db->table('admins')->where('admin_id',$adminID)->update($data);
        $param = array(
            'where' => ' AND admin_id="'.$adminID.'" ',
        );
        $result = $this->getAll($param);
        if ($result)
            $result = $result[0];

        return $result;
        
    }

    public function deleteAdmin($adminID, $deleteAdminID ){
        $data = [
            'admin_allowed_login'   => 0,
            'admin_delete_at'       => getCurrentTime(),
            'admin_delete_admin_id' => $deleteAdminID,
            'admin_active'          => 0
        ];

        $this->db->table('admins')->where('admin_id',$adminID)->update($data);
        $param = array(
            'where' => ' AND admin_id="'.$adminID.'" ',
        );
        $result = $this->getAll($param);
        if ($result)
            $result = $result[0];

        return $result;
        
    }

}