<?php
namespace App\Controllers;

use App\Models\CustomerModel;
use App\Models\OrderModel;

class Customer extends BaseController
{
	public function getAll()
	{
		if($this->request->isAjax()){
			return $this->data();
		}
		
		$data = [
			'title' => lang('Menu.master.customer.main'),
			'menu' => 'customer',
		];

		return view('customer/index', $data);
	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}
		
		$customerModel = new CustomerModel();

		$customerData = $customerModel->getAll();

		$data = [];

		if (@$customerData) {
        	foreach ($customerData as $index => $val ) {
        		
            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/customer/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

            $btn_edit = '&nbsp;<a href="'.base_url().'/customer/edit/'.$val->id.'" class="btn btn-success btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

            $value['no'] = $index+1;
            $value['type'] = $val->type;
            $value['name'] = $val->name;
            $value['address'] = $val->address;
            $value['code'] = $val->code;
            $value['bank'] = $val->bank;
            $value['bank_account_no'] = $val->bank_account_no;
            $value['bank_account_name'] = $val->bank_account_name;
            $value['phone'] = $val->phone;
            $value['insert_at'] = $val->insert_at;
            $value['action'] = $btn_edit.$btn_delete;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $customer = $this->request->getVar('customer');

        if (isset($customer[0]['column']) && $customer[0]['dir']) {
            $column = $customer[0]['column'];
            $dir    = $customer[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

	public function addCustomer()
	{
		if (!$this->request->isAjax()) {

			$data = [
				'title' => lang('Menu.master.customer.main'),
				'menu' => 'customer',
			];

			return view('customer/create', $data);
		}else{

			if (!$this->validate([
	            'type'             => ['label' => 'Tipe', 'rules' => 'required|max_length[255]'],
	            'name'             => ['label' => 'Nama', 'rules' => 'required|max_length[255]'],
	            'address'             => ['label' => 'Alamat', 'rules' => 'required|max_length[255]'],
	            'code'             => ['label' => 'Kode', 'rules' => 'required|max_length[255]|is_unique[customers.customer_code]'],
	            'bank'             => ['label' => 'Bank', 'rules' => 'required|max_length[255]'],
	            'bank_account_no'             => ['label' => 'No Akun Bank', 'rules' => 'required|max_length[255]'],
	            'bank_account_name'             => ['label' => 'Nama Akun Bank', 'rules' => 'required|max_length[255]'],
	            'email'             => ['label' => 'Email', 'rules' => 'max_length[255]'],
	            'phone'             => ['label' => 'No HP', 'rules' => 'required|max_length[255]'],
	        ])) {
				$msg = [
	                'error' => $this->validation->listErrors(),
	            ];
	            return json_encode($msg);
			}

			$data = [
				'customer_type'		     => $this->request->getPost('type'),
				'customer_name' 		=> $this->request->getPost('name'),
				'customer_address' 		    => $this->request->getPost('address'),
				'customer_code' 		=> $this->request->getPost('code'),
				'customer_bank' 		=> $this->request->getPost('bank'),
				'customer_bank_account_no' 		=> $this->request->getPost('bank_account_no'),
				'customer_bank_account_name' 		=> $this->request->getPost('bank_account_name'),
				'customer_email' 		=> $this->request->getPost('email'),
				'customer_phone' 		=> $this->request->getPost('phone'),
	            'customer_insert_admin_id' => $this->session->get('id'),
	            'customer_active'          => 1
	        ];

			$customerModel = new CustomerModel();

			$customerData = $customerModel->addCustomer($data);

			if ($customerData) {
				$data = [
					'success'=> lang('Validation.success_add'),
					'redirect'=> 'customer'
				];

				return json_encode($data);
			}

		}

	}

	public function updateCustomer($id)
	{

		if (!$this->request->isAjax()) {

			$customerModel = new CustomerModel();

			$dataCustomer = $customerModel->getByID($id);

			$data = [
				'data' => (isset($dataCustomer))? $dataCustomer : [],
				'title' => lang('Menu.master.customer.main'),
				'menu' => 'customer',
			];

			return view('customer/edit',  $data);

		}else{

			if (!$this->validate([
	            'type'             => ['label' => 'Tipe', 'rules' => 'required|max_length[255]'],
	            'name'             => ['label' => 'Nama', 'rules' => 'required|max_length[255]'],
	            'address'             => ['label' => 'Alamat', 'rules' => 'required|max_length[255]'],
	            'bank'             => ['label' => 'Bank', 'rules' => 'required|max_length[255]'],
	            'bank_account_no'             => ['label' => 'No Akun Bank', 'rules' => 'required|max_length[255]'],
	            'bank_account_name'             => ['label' => 'Nama Akun Bank', 'rules' => 'required|max_length[255]'],
	            'email'             => ['label' => 'Email', 'rules' => 'max_length[255]'],
	            'phone'             => ['label' => 'No HP', 'rules' => 'required|max_length[255]'],
	        ])) {
				$msg = [
	                'error' => $this->validation->listErrors(),
	            ];
	            return json_encode($msg);
			}

			$data = [
				'customer_type'		    => $this->request->getPost('type'),
				'customer_name' 		=> $this->request->getPost('name'),
				'customer_address' 		=> $this->request->getPost('address'),
				'customer_bank' 		=> $this->request->getPost('bank'),
				'customer_bank_account_no' 		=> $this->request->getPost('bank_account_no'),
				'customer_bank_account_name' 		=> $this->request->getPost('bank_account_name'),
				'customer_email' 		=> $this->request->getPost('email'),
				'customer_phone' 		=> $this->request->getPost('phone'),
	            'customer_update_admin_id' => $this->session->get('id'),
	            'customer_active'          => 1
	        ];

			$customerModel = new CustomerModel();

			$customerData = $customerModel->updateCustomer($data, $id);

			if ($customerData) {
				$data = [
					'success'=> lang('Validation.success_edit'),
					'redirect'=> 'customer'
				];

				return json_encode($data);
			}

		}
	}

	public function deleteCustomer($id)
	{

		$orderModel = new OrderModel();
		if(count($orderModel->getAll(["where" => ' AND order_customer_id="'.$id.'" AND order_active=1'])) > 0 ){
			$data = [
				'error'=> lang('Validation.customer_already_use'),
			];
			return json_encode($data);
		}

		$customerModel = new CustomerModel();

		$customerData = $customerModel->deleteCustomer($id, $this->session->get('id')); 

		if ($customerData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}

}