<?php

namespace App\Controllers;

use App\Models\GoldTypeModel;
use App\Models\ItemModel;

class GoldType extends BaseController
{
	public function getAll()
	{
		if($this->request->isAjax()){
			return $this->data();
		}
		
		$data = [
			'title' => lang('Menu.master.gold-type.main'),
			'menu' => 'gold-type',
		];

		return view('gold-type/index', $data);
	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}
		
		$goldTypeModel = new GoldTypeModel();

		$goldTypeData = $goldTypeModel->getAll(); 

		$data = [];

		if (@$goldTypeData) {
        	foreach ($goldTypeData as $index => $val ) {
        		
            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/gold-type/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

            $btn_edit = '&nbsp;<a href="'.base_url().'/gold-type/edit/'.$val->id.'" class="btn btn-success btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

            $value['no'] = $index+1;
            $value['code'] = $val->code;
            $value['name'] = $val->name;
            $value['insert_at'] = $val->insert_at;
            $value['action'] = $btn_edit.$btn_delete;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

	public function addGoldType()
	{
		if (!$this->request->isAjax()) {
			$data = [
				'title' => lang('Menu.master.gold-type.main'),
				'menu' => 'gold-type',
			];

			return view('gold-type/create', $data);
		}

		if (!$this->validate([
            'name'             => ['label' => 'Nama Warna Emas', 'rules' => 'required|max_length[255]'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$data = [
			'gt_code'		     => url_title($this->request->getPost('name'), "-", TRUE),
			'gt_name' 		     => $this->request->getPost('name'),
            'gt_insert_admin_id' => $this->session->get('id'),
            'gt_active'          => 1
        ];

		$goldTypeModel = new GoldTypeModel();

		$goldTypeData = $goldTypeModel->getByCode(url_title($this->request->getPost('name'), "-", TRUE));
        if (count($goldTypeData) > 0) {
				$msg = [
	                'error' => lang('Validation.name_already_exist'),
	            ];
	            return json_encode($msg);
        }

		$goldTypeData = $goldTypeModel->addGoldType($data); 
		

		if ($goldTypeData) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect'=> 'gold-type'
			];

			return json_encode($data);
		}

	}

	public function updateGoldType($id)
	{
		$goldTypeModel = new GoldTypeModel();

		$goldTypeData = $goldTypeModel->getByID($id);

		if (!$this->request->isAjax()) {
			$data = [
				'data' => (isset($goldTypeData))? $goldTypeData : [],
				'title' => lang('Menu.user.gold-type.main'),
				'menu' => 'gold-type',
			];

			return view('gold-type/edit',  $data);
		}

		if (!$this->validate([
            'name'             => ['label' => 'Nama Warna Emas', 'rules' => 'required|max_length[255]'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}


		$data = [
			'gt_code'		     => url_title($this->request->getPost('name'), "-", TRUE),
			'gt_name' 		 => $this->request->getPost('name'),
            'gt_update_admin_id' => $this->session->get('id'),
            'gt_active'          => 1
        ];

		if ($goldTypeData->name != $data['gt_name']) {
        	$goldTypeDataByCode = $goldTypeModel->getByCode(url_title($this->request->getPost('name'), "-", TRUE));
	        if (count($goldTypeDataByCode) > 0) {
					$msg = [
		                'error' => lang('Validation.name_already_exist'),
		            ];
		            return json_encode($msg);
	        }
        }

		$goldTypeModel = new GoldTypeModel();

		$goldTypeData = $goldTypeModel->updateGoldType($data, $id); 

		if ($goldTypeData) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'gold-type'
			];

			return json_encode($data);
		}
	}

	public function deleteGoldType($id)
	{
		$itemModel = new ItemModel();
		if(count($itemModel->getAll(["where" => ' AND item_gt_id="'.$id.'" AND item_active=1'])) > 0 ){
			$data = [
				'error'=> lang('Validation.gold_type_already_use'),
			];
			return json_encode($data);
		}

		$goldTypeModel = new GoldTypeModel();

		$goldTypeData = $goldTypeModel->deleteGoldType($id, $this->session->get('id')); 

		if ($goldTypeData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}

}