<?php
namespace App\Controllers;


use App\Models\OperatorModel;
use App\Models\OperatorTaskModel;
use Ramsey\Uuid\Uuid;
use App\Libraries\S3Library;
use App\Models\TaskModel;
use App\Models\AdminModel;


class Operator extends BaseController
{
	public function getAll()
	{
		if($this->request->isAjax()){
			return $this->data();

		}

		
		$data = [
			'title' => lang('Menu.master.operator.main'),
			'menu' => 'operator',
		];

		return view('operator/index', $data);
	}


	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}
		
		$operatorModel = new OperatorModel();
		$operatorData = $operatorModel->getAll();

		$data = [];

		if (@$operatorData) {
        	foreach ($operatorData as $index => $val ) {
        		
	            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/operator/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

	            $btn_edit = '&nbsp;<a href="'.base_url().'/operator/edit/'.$val->id.'" class="btn btn-warning btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

	            $value['name'] = $val->name;
	            $value['insert_at'] = $val->insert_at;
	            $value['no'] = $index+1;
 				$value['action'] = $btn_edit.$btn_delete;

	            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

	public function addOperator()
	{
		if (!$this->request->isAjax()) {

			$taskModel = new TaskModel();
			
			$data = [
				'title' => lang('Menu.master.operator.main'),
				'menu' => 'operator',
				'data_task' => $taskModel->getAll()
			];

			return view('operator/create', $data);
		}else{

			if (!$this->validate([
	            'name'             => ['label' => 'Nama Alur Kerja', 'rules' => 'required|max_length[255]'],
	            'task_id.0'             => ['label' => 'Divisi', 'rules' => 'required'],
	        ])) {
				$msg = [
	                'error' => $this->validation->listErrors(),
	            ];
	            return json_encode($msg);
			}

			$data = [
				'operator_name' 		     => $this->request->getPost('name'),
	            'operator_insert_admin_id' => $this->session->get('id'),
	            'operator_active'          => 1
	        ];

			$operatorModel = new OperatorModel();

	        $data2 = [];

			$operatorData = $operatorModel->addOperator($data); 
	        foreach ($this->request->getPost('task_id') as $key => $value) {
	        	$data2[] = [
	        		'ot_operator_id'=> $operatorData,
	        		'ot_task_id'=> $value,
	        		'ot_insert_admin_id'=> $this->session->get('id'),
	        		'ot_active' => 1
	        	];
	        }

			$operatorTaskModel = new OperatorTaskModel();

			$operatorTaskData = $operatorTaskModel->addOperatorTaskBulk($data2); 

			if ($operatorData) {
				$data = [
					'success'=> lang('Validation.success_add'),
					'redirect'=> 'operator'
				];

				return json_encode($data);
			}

		}

	}


	public function updateOperator($id)
	{
		$operatorModel = new OperatorModel();
		$operatorTaskModel = new OperatorTaskModel();

		$operatorData = $operatorModel->getByID($id);

		if (!$this->request->isAjax()) {

			$taskModel = new TaskModel();

			$data = [
				'data' => (isset($operatorData))? $operatorData : [],
				'title' => lang('Menu.master.operator.main'),
				'menu' => 'operator',
				'data_task' => $taskModel->getAll(),
				'data_operator_task'=> $operatorTaskModel->getOperatorTaskGetByOperator($id),
			];

			return view('operator/edit',  $data);
		}

		$data = [
			'operator_name' 		     => $this->request->getPost('name'),
            'operator_update_admin_id' => $this->session->get('id'),
            'operator_active'          => 1
        ];

		if (!$this->validate([
            'name'             => ['label' => 'Nama Alur Kerja', 'rules' => 'required|max_length[255]'],
            'task_id.0'             => ['label' => 'Divisi', 'rules' => 'required'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}


		$operatorData = $operatorModel->updateOperator($data, $id); 

		$data2 = [];
		foreach ($this->request->getPost('task_id') as $key => $value) {
        	$data2[] = [
        		'ot_operator_id'=> $id,
        		'ot_task_id'=> $value,
        		'ot_insert_admin_id'=> $this->session->get('id'),
        		'ot_active' => 1
        	];
        }

        $operatorTaskModel->deleteOperatorTaskByOperator($id, $this->session->get('id'));


		$operatorTaskData = $operatorTaskModel->addOperatorTaskBulk($data2); 

		if ($operatorData) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'operator'
			];

			return json_encode($data);
		}
	}

	public function deleteOperator($id)
	{

		$operatorModel = new OperatorModel();

		$operatorData = $operatorModel->deleteOperator($id, $this->session->get('id')); 

		if ($operatorData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}
	
}