<?php
namespace App\Controllers;

use App\Models\OrderModel;
use App\Models\OrderItemModel;
use App\Models\BatchItemModel;
use App\Models\CustomerModel;
use App\Models\WorkflowModel;
use App\Models\SizeModel;
use App\Models\GoldContentModel;
use App\Models\GoldTypeModel;
use App\Models\TreeModel;
use App\Models\CategoryModel;
use App\Models\TaskModel;
use App\Models\ItemModel;
use Ramsey\Uuid\Uuid;
use App\Libraries\S3Library;
use App\Models\OutgoingNoteModel;
use App\Models\OutgoingNoteDetailModel;
use App\Models\BatchItemOutputModel;

class Report extends BaseController
{
    public function order()
    {
        if($this->request->isAjax()){
            return $this->dataOrder();
        }

        if ($this->request->getGet('export-excel') == true) {
            return $this->exportOrder();
        }

        $workflowModel = new WorkflowModel();
        $customerModel = new CustomerModel();
        $sizeModel = new SizeModel();
        $goldContentModel = new GoldContentModel();
        $goldTypeModel = new GoldTypeModel();
        $categoryModel = new CategoryModel();
        $taskModel = new TaskModel();
        $itemModel = new ItemModel();
        $treeModel = new TreeModel();

        $data = [
            'data_workflow'=> $workflowModel->getAll(),
            'data_customer'=> $customerModel->getAll(),
            'data_size'=> $sizeModel->getAll(),
            'data_gold_content'=> $goldContentModel->getAll(),
            'data_gold_type'=> $goldTypeModel->getAll(),
            'data_category'=> $categoryModel->getAll(),
            'data_task'=> $taskModel->getAll(),
            'data_item'=> $itemModel->getAll(),
            'data_tree'=> $treeModel->getAll(),
            'title' => lang('Menu.report.order.main'),
            'menu' => 'order-report',
        ];

        return view('report/order', $data);
    }

    public function dataOrder()
    {
        if (!$this->request->isAjax()) {
            return false;
        }

        $fromDate = $this->request->getGet('from_date');
        $toDate = $this->request->getGet('to_date');
        $workflowId = $this->request->getGet('workflow_id');
        $customerId = $this->request->getGet('customer_id');
        $sizeId = $this->request->getGet('size_id');
        $gcId = $this->request->getGet('gc_id');
        $gtId = $this->request->getGet('gt_id');
        $categoryId = $this->request->getGet('category_id');
        $taskId = $this->request->getGet('task_id');
        $itemSKU = $this->request->getGet('item_sku');
        $treeId = $this->request->getGet('tree_id');
        $status = $this->request->getGet('status');
        $orderNote = $this->request->getGet('note');

        $queryWhere = '';

        if ($fromDate != '' && $toDate != '') {
            $queryWhere .= ' AND order_date >="'.$fromDate.'" ';
            $queryWhere .= ' AND order_date <= "'.$toDate.'" ';
        }else{
            $queryWhere .= ' AND MONTH(order_date)="'.date('m').'" AND YEAR(order_date)="'.date('Y').'" ';
        }

        if ($workflowId != '') {
            $queryWhere .= ' AND item_workflow_id = "'.$workflowId.'" ';
        }

        if ($customerId != '') {
            $queryWhere .= ' AND order_customer_id = "'.$customerId.'" ';
        }

        if ($sizeId != '') {
            $queryWhere .= ' AND item_size_id = "'.$sizeId.'" ';
        }

        if ($gcId != '') {
            $queryWhere .= ' AND item_gc_id = "'.$gcId.'" ';
        }

        if ($gtId != '') {
            $queryWhere .= ' AND item_gt_id = "'.$gtId.'" ';
        }

        if ($categoryId != '') {
            $queryWhere .= ' AND item_category_id = "'.$categoryId.'" ';
        }

        if ($taskId != '') {
            $queryWhere .= ' AND wt_task_id = "'.$taskId.'" ';
        }

        if ($itemSKU != '') {
            $queryWhere .= ' AND items.item_sku = "'.$itemSKU.'" ';
        }

        if ($status != '') {
            $queryWhere .= ' AND orders.order_status = "'.$status.'" ';
        }

        if ($orderNote != '') {
            $queryWhere .= ' AND orders.order_note = "'.$orderNote.'" ';
        }

        $orderItemModel = new OrderItemModel();

        $orderItemData = $orderItemModel->getAll(['where'=> $queryWhere, 'orderby'=> ' oi_insert_at DESC ']);

        $data = [];

        if (@$orderItemData) {
            foreach ($orderItemData as $index => $val ) {

            $value['no'] = ($index+1);
            $value['date'] = $val->date;
            $value['due_date'] = $val->due_date;
            $value['start_date'] = $val->start_date;
            $value['name'] = $val->name;
            $value['sku'] = $val->sku;
            $value['note'] = '<a href="'.base_url('batch-item-output?order_item='.$val->id).'">'.$val->order_note.'</a>';
            $value['customer_name'] = $val->customer_name;
            $value['customer_phone'] = $val->customer_phone;
            $value['customer_address'] = $val->customer_address;
            $value['quantity'] = $val->quantity;
            $value['completed_quantity'] = $val->completed_quantity;
            $value['completed_weight'] = $val->completed_weight;
            $value['status'] = $val->order_item_status;
            $value['category'] = $val->category_name;
            $value['workflow'] = $val->workflow_name;
            $value['gold_content'] = $val->gc_name;
            $value['gold_type'] = $val->gt_name;
            $value['weight'] = $val->weight;
            $value['size'] = $val->size_name;
            $value['use_wire'] = $val->use_wire;
            $value['stone_type'] = $val->stone_type;
            $value['print_no'] = $val->print_no;

            $value['start_due_date'] = $val->start_due_date;
            $value['request_total'] = $val->request_total;
            $value['request_description'] = $val->request_description;

            $value['user_update_id'] = $val->user_update;
            $value['user_insert_id'] = $val->user_insert;
            $value['user_delete_id'] = $val->user_delete;

            array_push($data,$value);
            }
        }

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

    public function exportOrder(){
        $fromDate = $this->request->getGet('from_date');
        $toDate = $this->request->getGet('to_date');
        $workflowId = $this->request->getGet('workflow_id');
        $customerId = $this->request->getGet('customer_id');
        $sizeId = $this->request->getGet('size_id');
        $gcId = $this->request->getGet('gc_id');
        $gtId = $this->request->getGet('gt_id');
        $categoryId = $this->request->getGet('category_id');
        $taskId = $this->request->getGet('task_id');
        $itemSKU = $this->request->getGet('item_sku');
        $treeId = $this->request->getGet('tree_id');
        $status = $this->request->getGet('status');
        $orderNote = $this->request->getGet('note');

        $queryWhere = '';

        if ($fromDate != '' && $toDate != '') {
            $queryWhere .= ' AND order_date >="'.$fromDate.'" ';
            $queryWhere .= ' AND order_date <= "'.$toDate.'" ';
        }else{
            $queryWhere .= ' AND MONTH(order_date)="'.date('m').'" AND YEAR(order_date)="'.date('Y').'" ';
        }

        if ($workflowId != '') {
            $queryWhere .= ' AND item_workflow_id = "'.$workflowId.'" ';
        }

        if ($customerId != '') {
            $queryWhere .= ' AND order_customer_id = "'.$customerId.'" ';
        }

        if ($sizeId != '') {
            $queryWhere .= ' AND item_size_id = "'.$sizeId.'" ';
        }

        if ($gcId != '') {
            $queryWhere .= ' AND item_gc_id = "'.$gcId.'" ';
        }

        if ($gtId != '') {
            $queryWhere .= ' AND item_gt_id = "'.$gtId.'" ';
        }

        if ($categoryId != '') {
            $queryWhere .= ' AND item_category_id = "'.$categoryId.'" ';
        }

        if ($taskId != '') {
            $queryWhere .= ' AND wt_task_id = "'.$taskId.'" ';
        }

        if ($itemSKU != '') {
            $queryWhere .= ' AND items.item_sku = "'.$itemSKU.'" ';
        }

        if ($status != '') {
            $queryWhere .= ' AND orders.order_status = "'.$status.'" ';
        }

        if ($orderNote != '') {
            $queryWhere .= ' AND orders.order_note = "'.$orderNote.'" ';
        }

        $orderItemModel = new OrderItemModel();

        $orderItemData = $orderItemModel->getAll(['where'=> $queryWhere, 'orderby'=> ' oi_insert_at DESC ']);

        $data = [
            'title' => lang('Menu.report.order.main').'-'.date('Ymd'),
            'data' => $orderItemData,
        ];

        return view('report/export-order', $data);
    }


    public function batch()
    {
        if($this->request->isAjax()){
            return $this->dataBatch();
        }

        if ($this->request->getGet('export-excel') == true) {
            return $this->exportBatch();
        }

        $workflowModel = new WorkflowModel();
        $customerModel = new CustomerModel();
        $sizeModel = new SizeModel();
        $goldContentModel = new GoldContentModel();
        $goldTypeModel = new GoldTypeModel();
        $categoryModel = new CategoryModel();
        $taskModel = new TaskModel();
        $itemModel = new ItemModel();
        $treeModel = new TreeModel();

        $data = [
            'data_workflow'=> $workflowModel->getAll(),
            'data_customer'=> $customerModel->getAll(),
            'data_size'=> $sizeModel->getAll(),
            'data_gold_content'=> $goldContentModel->getAll(),
            'data_gold_type'=> $goldTypeModel->getAll(),
            'data_category'=> $categoryModel->getAll(),
            'data_task'=> $taskModel->getAll(),
            'data_item'=> $itemModel->getAll(),
            'data_tree'=> $treeModel->getAll(),
            'title' => lang('Menu.report.batch.main'),
            'menu' => 'batch-report',
        ];

        return view('report/batch', $data);
    }

    public function dataBatch()
    {
        if (!$this->request->isAjax()) {
            return false;
        }


        $fromDate = $this->request->getGet('from_date');
        $toDate = $this->request->getGet('to_date');
        $workflowId = $this->request->getGet('workflow_id');
        $customerId = $this->request->getGet('customer_id');
        $sizeId = $this->request->getGet('size_id');
        $gcId = $this->request->getGet('gc_id');
        $gtId = $this->request->getGet('gt_id');
        $categoryId = $this->request->getGet('category_id');
        $taskId = $this->request->getGet('task_id');
        $itemSKU = $this->request->getGet('item_sku');
        $treeId = $this->request->getGet('tree_id');
        $status = $this->request->getGet('status');
        $batchCode = $this->request->getGet('batch_code');
        $orderNote = $this->request->getGet('note');

        $queryWhere = '';

        if ($fromDate != '' && $toDate != '') {
            $queryWhere .= ' AND batch_start_date >="'.$fromDate.'" ';
            $queryWhere .= ' AND batch_start_date <= "'.$toDate.'" ';
        }else{
            $queryWhere .= ' AND MONTH(batch_start_date)="'.date('m').'" AND YEAR(batch_start_date)="'.date('Y').'" ';
        }

        if ($workflowId != '') {
            $queryWhere .= ' AND item_workflow_id = "'.$workflowId.'" ';
        }

        if ($customerId != '') {
            $queryWhere .= ' AND order_customer_id = "'.$customerId.'" ';
        }

        if ($sizeId != '') {
            $queryWhere .= ' AND item_size_id = "'.$sizeId.'" ';
        }

        if ($gcId != '') {
            $queryWhere .= ' AND item_gc_id = "'.$gcId.'" ';
        }

        if ($gtId != '') {
            $queryWhere .= ' AND item_gt_id = "'.$gtId.'" ';
        }

        if ($categoryId != '') {
            $queryWhere .= ' AND item_category_id = "'.$categoryId.'" ';
        }

        if ($taskId != '') {
            $queryWhere .= ' AND wt_task_id = "'.$taskId.'" ';
        }

        if ($itemSKU != '') {
            $queryWhere .= ' AND items.item_sku = "'.$itemSKU.'" ';
        }

        if ($status != '') {
            $queryWhere .= ' AND batches.batch_status = "'.$status.'" ';
        }

        if ($batchCode != '') {
            $queryWhere .= ' AND batches.batch_code = "'.$batchCode.'" ';
        }

        if ($orderNote != '') {
            $queryWhere .= ' AND orders.order_note = "'.$orderNote.'" ';
        }
        
        $batchItemModel = new BatchItemModel();

        $batchItemData = $batchItemModel->getAll(['where'=> $queryWhere, 'orderby'=> ' bi_insert_at DESC ']);

        $data = [];

        if (@$batchItemData) {
            foreach ($batchItemData as $index => $val ) {

            $value['no'] = ($index+1);
            $value['start_date'] = $val->start_date;
            $value['note'] = $val->order_note;
            $value['batch_code'] = $val->batch_code;
            $value['name'] = $val->name;
            $value['sku'] = $val->sku;
            $value['customer_name'] = $val->customer_name;
            $value['customer_phone'] = $val->customer_phone;
            $value['customer_address'] = $val->customer_address;
            $value['category'] = $val->category_name;
            $value['gold_content'] = $val->gc_name;
            $value['quantity'] = $val->quantity;
            $value['completed_quantity'] = $val->completed_quantity;
            $value['completed_weight'] = $val->completed_weight;
            $value['status'] = $val->batch_status;
            $value['remark'] = $val->batch_remark;
            $value['weight'] = $val->weight;
            $value['size'] = $val->size_name;
            $value['use_wire'] = $val->use_wire;
            $value['stone_type'] = $val->stone_type;
            $value['print_no'] = $val->print_no;
            $value['user_update_id'] = $val->user_update;
            $value['user_insert_id'] = $val->user_insert;
            $value['user_delete_id'] = $val->user_delete;

            $value['start_due_date'] = $val->start_due_date;
            $value['request_total'] = $val->request_total;
            $value['request_description'] = $val->request_description;

            array_push($data,$value);
            }
        }

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

    public function exportBatch(){
        $fromDate = $this->request->getGet('from_date');
        $toDate = $this->request->getGet('to_date');
        $workflowId = $this->request->getGet('workflow_id');
        $customerId = $this->request->getGet('customer_id');
        $sizeId = $this->request->getGet('size_id');
        $gcId = $this->request->getGet('gc_id');
        $gtId = $this->request->getGet('gt_id');
        $categoryId = $this->request->getGet('category_id');
        $taskId = $this->request->getGet('task_id');
        $itemSKU = $this->request->getGet('item_sku');
        $treeId = $this->request->getGet('tree_id');
        $status = $this->request->getGet('status');
        $batchCode = $this->request->getGet('batch_code');
        $orderNote = $this->request->getGet('note');

        $queryWhere = '';

        if ($fromDate != '' && $toDate != '') {
            $queryWhere .= ' AND batch_start_date >="'.$fromDate.'" ';
            $queryWhere .= ' AND batch_start_date <= "'.$toDate.'" ';
        }else{
            $queryWhere .= ' AND MONTH(batch_start_date)="'.date('m').'" AND YEAR(batch_start_date)="'.date('Y').'" ';
        }

        if ($workflowId != '') {
            $queryWhere .= ' AND item_workflow_id = "'.$workflowId.'" ';
        }

        if ($customerId != '') {
            $queryWhere .= ' AND order_customer_id = "'.$customerId.'" ';
        }

        if ($sizeId != '') {
            $queryWhere .= ' AND item_size_id = "'.$sizeId.'" ';
        }

        if ($gcId != '') {
            $queryWhere .= ' AND item_gc_id = "'.$gcId.'" ';
        }

        if ($gtId != '') {
            $queryWhere .= ' AND item_gt_id = "'.$gtId.'" ';
        }

        if ($categoryId != '') {
            $queryWhere .= ' AND item_category_id = "'.$categoryId.'" ';
        }

        if ($taskId != '') {
            $queryWhere .= ' AND wt_task_id = "'.$taskId.'" ';
        }

        if ($itemSKU != '') {
            $queryWhere .= ' AND items.item_sku = "'.$itemSKU.'" ';
        }

        if ($status != '') {
            $queryWhere .= ' AND batches.batch_status = "'.$status.'" ';
        }

        if ($batchCode != '') {
            $queryWhere .= ' AND batches.batch_code = "'.$batchCode.'" ';
        }

        if ($orderNote != '') {
            $queryWhere .= ' AND orders.order_note = "'.$orderNote.'" ';
        }

        $batchItemModel = new BatchItemModel();

        $batchItemData = $batchItemModel->getAll(['where'=> $queryWhere, 'orderby'=> ' bi_insert_at DESC ']);

        $data = [
            'title' => lang('Menu.report.batch.main').'-'.date('Ymd'),
            'data' => $batchItemData,
        ];

        return view('report/export-batch', $data);
    }



    public function outgoingNote()
    {
        if($this->request->isAjax()){
            return $this->dataOutgoingNote();
        }

        if ($this->request->getGet('export-excel') == true) {
            return $this->exportOutgoingNote();
        }

        $taskModel = new TaskModel();
        
        $data = [
            'title' => lang('Menu.report.outgoing-note.main'),
            'menu' => 'outgoing-note-report',
            'data_task' => $taskModel->getAll()
        ];

        return view('report/outgoing-note', $data);
    }
    
    public function dataOutgoingNote()
    {
        if (!$this->request->isAjax()) {
            return false;
        }

        $fromDate = $this->request->getVar('from_date', FILTER_SANITIZE_STRING);
        $toDate = $this->request->getVar('to_date', FILTER_SANITIZE_STRING);
        $notes = $this->request->getVar('notes', FILTER_SANITIZE_STRING);
        $taskID = $this->request->getVar('task_id', FILTER_SANITIZE_STRING);
        $batchCode = $this->request->getVar('batch_code', FILTER_SANITIZE_STRING);
        $orderNote = $this->request->getVar('order_note', FILTER_SANITIZE_STRING);

        $queryWhere = '';

        if ($fromDate != '' && $toDate != '') {
            $queryWhere .= ' AND on_date >="'.$fromDate.'" ';
            $queryWhere .= ' AND on_date <= "'.$toDate.'" ';
        }else{
            $queryWhere .= ' AND MONTH(on_date)="'.date('m').'" AND YEAR(on_date)="'.date('Y').'" ';
        }

        if ($batchCode != '') {
            $queryWhere .= ' AND batch_code = "'.$batchCode.'" ';
        }

        if ($orderNote != '') {
            $queryWhere .= ' AND order_note= "'.$orderNote.'" ';
        }

        if ($taskID != '') {
            if ($taskID == '-') {
                $queryWhere .= ' AND task_id IS NULL ';
            }else{
                $queryWhere .= ' AND task_id= "'.$taskID.'" ';
            }
        }

        if ($notes != '') {
            $queryWhere .= ' AND on_notes= "'.$notes.'" ';
        }

        
        $outgoingNoteDetailModel = new OutgoingNoteDetailModel();
        $outgoingNoteDetailData = $outgoingNoteDetailModel->getAll([ 
            'where' => $queryWhere.' AND on_types="Bahan Sisa"',
            'orderby'=> 'on_insert_at DESC'
        ]);

        $data = [];

        if (@$outgoingNoteDetailData) {
            foreach ($outgoingNoteDetailData as $index => $val ) {

                $value['notes'] = $val->on_notes;
                $value['remark'] = $val->on_remark;
                $value['date'] =  $val->on_date;
                $value['start_date'] = $val->on_start_date;
                $value['end_date'] = $val->on_end_date;
                $value['remark'] = $val->on_remark;
                $value['order_note'] = $val->order_note;
                $value['batch_code'] = $val->batch_code;
                $value['group_note'] = $val->group_note;
                $value['quantity'] = $val->quantity;
                $value['weight'] = $val->weight;
                $value['task_name'] = $val->task_name;
                $value['status'] = $val->on_status;
                $value['insert_at'] = $val->insert_at;
                $value['insert_admin_fullname'] = $val->insert_admin_fullname;

                array_push($data,$value);
            }
        }

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }



    public function exportOutgoingNote()
    {

        $fromDate = $this->request->getVar('from_date', FILTER_SANITIZE_STRING);
        $toDate = $this->request->getVar('to_date', FILTER_SANITIZE_STRING);
        $notes = $this->request->getVar('notes', FILTER_SANITIZE_STRING);
        $taskID = $this->request->getVar('task_id', FILTER_SANITIZE_STRING);
        $batchCode = $this->request->getVar('batch_code', FILTER_SANITIZE_STRING);
        $orderNote = $this->request->getVar('order_note', FILTER_SANITIZE_STRING);

        $queryWhere = '';

        if ($fromDate != '' && $toDate != '') {
            $queryWhere .= ' AND on_date >="'.$fromDate.'" ';
            $queryWhere .= ' AND on_date <= "'.$toDate.'" ';
        }else{
            $queryWhere .= ' AND MONTH(on_date)="'.date('m').'" AND YEAR(on_date)="'.date('Y').'" ';
        }

        if ($batchCode != '') {
            $queryWhere .= ' AND batch_code = "'.$batchCode.'" ';
        }

        if ($orderNote != '') {
            $queryWhere .= ' AND order_note= "'.$orderNote.'" ';
        }

        if ($taskID != '') {
            if ($taskID == '-') {
                $queryWhere .= ' AND task_id IS NULL ';
            }else{
                $queryWhere .= ' AND task_id= "'.$taskID.'" ';
            }
        }

        if ($notes != '') {
            $queryWhere .= ' AND on_notes= "'.$notes.'" ';
        }

        
        $outgoingNoteDetailModel = new OutgoingNoteDetailModel();
        $outgoingNoteDetailData = $outgoingNoteDetailModel->getAll([ 
            'where' => $queryWhere,
            'orderby'=> 'on_insert_at DESC'
        ]);

        $data = [
            'title' => lang('Menu.order.outgoing-note.main'),
            'menu' => 'outgoing-note-report',
            'data' => $outgoingNoteDetailData
        ];

        return view('report/export-outgoing-note', $data);

    }







    public function outgoingBatch()
    {
        if($this->request->isAjax()){
            return $this->dataOutgoingBatch();
        }

        if ($this->request->getGet('export-excel') == true) {
            return $this->exportOutgoingBatch();
        }
        
        $data = [
            'title' => lang('Menu.report.outgoing-batch.main'),
            'menu' => 'outgoing-batch-report',
        ];

        return view('report/outgoing-batch', $data);
    }
    
    public function dataOutgoingBatch()
    {
        // if (!$this->request->isAjax()) {
        //     return false;
        // }

        $fromDate = $this->request->getVar('from_date', FILTER_SANITIZE_STRING);
        $toDate = $this->request->getVar('to_date', FILTER_SANITIZE_STRING);
        $notes = $this->request->getVar('notes', FILTER_SANITIZE_STRING);
        $taskID = $this->request->getVar('task_id', FILTER_SANITIZE_STRING);
        $batchCode = $this->request->getVar('batch_code', FILTER_SANITIZE_STRING);
        $orderNote = $this->request->getVar('order_note', FILTER_SANITIZE_STRING);

        $queryWhere = '';

        if ($fromDate != '' && $toDate != '') {
            $queryWhere .= ' AND on_date >="'.$fromDate.'" ';
            $queryWhere .= ' AND on_date <= "'.$toDate.'" ';
        }else{
            $queryWhere .= ' AND MONTH(on_date)="'.date('m').'" AND YEAR(on_date)="'.date('Y').'" ';
        }

        if ($batchCode != '') {
            $queryWhere .= ' AND batch_code = "'.$batchCode.'" ';
        }

        if ($orderNote != '') {
            $queryWhere .= ' AND order_note= "'.$orderNote.'" ';
        }

        if ($taskID != '') {
            if ($taskID == '-') {
                $queryWhere .= ' AND task_id IS NULL ';
            }else{
                $queryWhere .= ' AND task_id= "'.$taskID.'" ';
            }
        }

        if ($notes != '') {
            $queryWhere .= ' AND on_notes= "'.$notes.'" ';
        }

        
        $outgoingNoteDetailModel = new OutgoingNoteDetailModel();
        $outgoingNoteDetailData = $outgoingNoteDetailModel->getAll([ 
            'where' => $queryWhere.' AND on_types="Barang Baik" ',
            'orderby'=> 'on_insert_at DESC'
        ]);

        $data = [];

        if (@$outgoingNoteDetailData) {
            foreach ($outgoingNoteDetailData as $index => $val ) {

                $value['notes'] = $val->on_notes;
                $value['remark'] = $val->on_remark;
                $value['date'] =  $val->on_date;
                $value['start_date'] = $val->on_start_date;
                $value['end_date'] = $val->on_end_date;
                $value['remark'] = $val->on_remark;
                $value['order_note'] = $val->order_note;
                $value['batch_code'] = $val->batch_code;
                $value['group_note'] = $val->group_note;
                $value['quantity'] = $val->quantity;
                $value['weight'] = $val->weight;
                $value['task_name'] = $val->task_name;
                $value['status'] = $val->on_status;
                $value['insert_at'] = $val->insert_at;
                $value['insert_admin_fullname'] = $val->insert_admin_fullname;

                array_push($data,$value);
            }
        }

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }



    public function exportOutgoingBatch()
    {

        $fromDate = $this->request->getVar('from_date', FILTER_SANITIZE_STRING);
        $toDate = $this->request->getVar('to_date', FILTER_SANITIZE_STRING);
        $notes = $this->request->getVar('notes', FILTER_SANITIZE_STRING);
        $taskID = $this->request->getVar('task_id', FILTER_SANITIZE_STRING);
        $batchCode = $this->request->getVar('batch_code', FILTER_SANITIZE_STRING);
        $orderNote = $this->request->getVar('order_note', FILTER_SANITIZE_STRING);

        $queryWhere = '';

        if ($fromDate != '' && $toDate != '') {
            $queryWhere .= ' AND on_date >="'.$fromDate.'" ';
            $queryWhere .= ' AND on_date <= "'.$toDate.'" ';
        }else{
            $queryWhere .= ' AND MONTH(on_date)="'.date('m').'" AND YEAR(on_date)="'.date('Y').'" ';
        }

        if ($batchCode != '') {
            $queryWhere .= ' AND batch_code = "'.$batchCode.'" ';
        }

        if ($orderNote != '') {
            $queryWhere .= ' AND order_note= "'.$orderNote.'" ';
        }

        if ($taskID != '') {
            if ($taskID == '-') {
                $queryWhere .= ' AND task_id IS NULL ';
            }else{
                $queryWhere .= ' AND task_id= "'.$taskID.'" ';
            }
        }

        if ($notes != '') {
            $queryWhere .= ' AND on_notes= "'.$notes.'" ';
        }

        
        $outgoingNoteDetailModel = new OutgoingNoteDetailModel();
        $outgoingNoteDetailData = $outgoingNoteDetailModel->getAll([ 
            'where' => $queryWhere,
            'orderby'=> 'on_insert_at DESC'
        ]);

        $data = [
            'title' => lang('Menu.order.outgoing-batch.main'),
            'menu' => 'outgoing-batch-report',
            'data' => $outgoingNoteDetailData
        ];

        return view('report/export-outgoing-batch', $data);

    }


    public function production()
    {
        if($this->request->isAjax()){

            $paramType = $this->request->getGet('param');
            if ($paramType == 'production2') {
                return $this->dataProduction2();
            }else{
                return $this->dataProduction();
            }
        }

        if ($this->request->getGet('export-excel') == true) {
            return $this->exportProduction();
        }

        $data = [
            'title' => lang('Menu.report.production.main'),
            'menu' => 'production-report',
        ];

        return view('report/production', $data);
    }

    public function dataProduction()
    {
        // if (!$this->request->isAjax()) {
        //     return false;
        // }

        $fromDate = $this->request->getGet('from_date');
        $toDate = $this->request->getGet('to_date');
        
        $queryWhere = '';


        if ($fromDate != '' && $toDate != '') {
            $queryWhere .= ' AND batch_start_date >="'.$fromDate.'" ';
            $queryWhere .= ' AND batch_start_date <= "'.$toDate.'" ';
        }else{
            $queryWhere .= ' AND MONTH(batch_start_date)="'.date('m').'" AND YEAR(batch_start_date)="'.date('Y').'" ';
        }

        $batchItemModel = new BatchItemModel();

        $batchItemData = $batchItemModel->getAllProduction(['select' => ' SUM(order_items.oi_quantity) as quantity, CEIL(batches.batch_total_gold_weight) as weight, DATE(batch_start_date) as "date" ', 'where'=> $queryWhere. ' AND (batch_status="Completed" OR batch_status="Proses") ', 'orderby'=> ' bi_insert_at DESC ', 'groupby' => ' DATE(batch_start_date) HAVING quantity != "" ' ]);

        $data = [];

        if (@$batchItemData) {
            $quantityTotal = 0;
            $weightTotal = 0;

            foreach ($batchItemData as $index => $val ) {

                $value['no'] = ($index+1);
                $value['date'] = $val->date;
                $value['quantity'] = $val->quantity;
                $value['weight'] = $val->weight;

                array_push($data,$value);

                $quantityTotal += $val->quantity;
                $weightTotal += $val->weight;
            }

            $value['no'] = '';
            $value['date'] = '<strong>Total</strong>';
            $value['quantity'] = '<strong>'.round($quantityTotal, 2).'</strong>';
            $value['weight'] = '<strong>'.round($weightTotal, 2).'</strong>';

            array_push($data,$value);
        }

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

    public function dataProduction2()
    {
        // if (!$this->request->isAjax()) {
        //     return false;
        // }

        $fromDate = $this->request->getGet('from_date');
        $toDate = $this->request->getGet('to_date');

        if ($fromDate == '' || $toDate == '') {
            
            $result = [
                'recordsTotal'    => 0,
                'recordsFiltered' => 0,
                'data'            => [],
            ];
            return json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
        }

        $queryWhere = '';
        

        if ($fromDate != '' && $toDate != '') {
            $queryWhere .= ' AND DATE(bio_insert_at) >="'.$fromDate.'" ';
            $queryWhere .= ' AND DATE(bio_insert_at) <= "'.$toDate.'" ';
        }else{
            $queryWhere .= ' AND MONTH(bio_insert_at)="'.date('m').'" AND YEAR(bio_insert_at)="'.date('Y').'" ';
        }

        $batchItemOutputModel = new BatchItemOutputModel();

        $batchItemOutputData = $batchItemOutputModel->getAllReportProduksi(['where'=> $queryWhere.' AND bio_active=1 AND bio_is_melt=0 ', 'groupby' => 'DATE(bio_insert_at), gc_id ' ]);

        $data = [];

        if (@$batchItemOutputData) {

            $totalTotalWeight = 0;
            $totalTotalQuantity = 0;
            $totalTotalWeight24 = 0;

            foreach ($batchItemOutputData as $index => $val ) {
                $value['no'] = ($index+1);
                $value['date'] = $val->date;
                $value['content'] = $val->gc_name;
                $value['total_weight'] = $val->total_weight;
                $value['total_quantity'] = $val->total_quantity;
                $value['total_weight_24'] = round($val->total_weight*($val->gc_code/100), 2);

                array_push($data,$value);

                $totalTotalWeight += $val->total_weight;
                $totalTotalQuantity += $val->total_quantity;
                $totalTotalWeight24 += round($val->total_weight*($val->gc_code/100), 2);
            }

            $value['no'] = '';
            $value['date'] = '';
            $value['content'] = '<strong>Total</strong>';
            $value['total_weight'] = '<strong>'.round($totalTotalWeight, 2).'</strong>';
            $value['total_quantity'] = '<strong>'.round($totalTotalQuantity, 2).'</strong>';
            $value['total_weight_24'] = '<strong>'.round($totalTotalWeight24, 2).'</strong>';

            array_push($data,$value);
        }

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

    public function exportProduction(){

        $fromDate = $this->request->getGet('from_date');
        $toDate = $this->request->getGet('to_date');
        $type = $this->request->getGet('type');

        if ($type == "data") {
            
            $queryWhere = '';
            if ($fromDate != '' && $toDate != '') {
                $queryWhere .= ' AND batch_start_date >="'.$fromDate.'" ';
                $queryWhere .= ' AND batch_start_date <= "'.$toDate.'" ';
            }else{
                $queryWhere .= ' AND MONTH(batch_start_date)="'.date('m').'" AND YEAR(batch_start_date)="'.date('Y').'" ';
            }

            $batchItemModel = new BatchItemModel();

            $batchItemData = $batchItemModel->getAllProduction(['select' => ' SUM(order_items.oi_quantity) as quantity, CEIL(batches.batch_total_gold_weight) as weight, DATE(batch_start_date) as "date" ', 'where'=> $queryWhere. ' AND (batch_status="Completed" OR batch_status="Proses") ', 'orderby'=> ' bi_insert_at DESC ', 'groupby' => ' DATE(batch_start_date) HAVING quantity != "" ' ]);

             $data = [
                'title' => lang('Menu.report.production.main').'- Permintaan Bahan -'.date('Ymd'),
                'data' => $batchItemData,
                'type' => $type,
            ];

        }else{

            $queryWhere = '';
            if ($fromDate != '' && $toDate != '') {
                $queryWhere .= ' AND DATE(bio_insert_at) >="'.$fromDate.'" ';
                $queryWhere .= ' AND DATE(bio_insert_at) <= "'.$toDate.'" ';
            }else{
                $queryWhere .= ' AND MONTH(bio_insert_at)="'.date('m').'" AND YEAR(bio_insert_at)="'.date('Y').'" ';
            }

            $batchItemOutputModel = new BatchItemOutputModel();

            $batchItemData2 = $batchItemOutputModel->getAllReportProduksi(['where'=> $queryWhere.' AND bio_active=1 AND bio_is_melt=0 ', 'groupby' => 'DATE(bio_insert_at), gc_id ' ]);
             $data = [
                'title' => lang('Menu.report.production.main').'- Hasil Produksi -'.date('Ymd'),
                'data2' => $batchItemData2,
                'type' => $type,
            ];
        }


        return view('report/export-production', $data);
    }


}