<?php 
namespace App\Libraries;

use Aws\S3\S3Client;

class S3Library {
    private $s3Bucket;
    private $region;
    private $key;
    private $secret;

    public function __construct(){
        $this->s3Bucket = getenv('AWS_BUCKET');
        $this->region = getenv('AWS_REGION');
        $this->key  = getenv('AWS_SECRET_ID');
        $this->secret = getenv('AWS_SECRET_KEY');
        $this->request = \Config\Services::request();
    }

    public function putObject($path, $fileData){
        $fileName = $fileData->getClientName();
        $fileExtension = $fileData->guessExtension();
        $fileType = $fileData->getMimeType();

        $s3Client = new S3Client([
            'version'       => 'latest',
            'region'        => $this->region,
            'credentials'   => [
                'key'           => $this->key,
                'secret'        => $this->secret,
            ]
        ]);
        
        try{
            $data = [
                'Bucket' => $this->s3Bucket,
                'Key'    => $path,
                'SourceFile'   => $fileData,
                'ContentType' => $fileType,
                'ACL'    => 'public-read',
            ];
            $result = $s3Client->putObject($data);
            return true;
        }catch(Aws\S3\Exception\S3Exception $e){
            return false;
        }


    }

    public function deleteObject($path){
        try{
            $s3Client = new S3Client([
                'version'       => 'latest',
                'region'        => $this->region,
                'credentials'   => [
                    'key'           => $this->key,
                    'secret'        => $this->secret,
                ]
            ]);
            
            $data = [
                'Bucket' => $this->s3Bucket,
                'Key'   => $path
            ];
            $s3Client->deleteObject($data);
            return true;
        }catch(Aws\S3\Exception\S3Exception $e){
            return false;
        }
    }
}