<?php
namespace App\Controllers;

use Ramsey\Uuid\Uuid;
use App\Libraries\S3Library;
use App\Libraries\Barcode;

use App\Models\BatchItemOutputModel;
use App\Models\BatchItemModel;
use App\Models\BatchModel;


class BatchItemOutput extends BaseController
{
	public function index()
	{
		if($this->request->isAjax()){
			return $this->data();
		}

		$data = [
			'title' => lang('Menu.order.batch-item-output.main'),
			'menu' => 'batch-item-output',
		];

		return view('batch-item-output/index', $data);
	}

	public function store(){

		$rules = [
            'item_quantity' => ['label' => 'Jumlah', 'rules' => 'required|max_length[11]'],
            'item_weight' => ['label' => 'Weight', 'rules' => 'required|max_length[65]'],
        ];

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$isMelt = $this->request->getVar('is_melt');
		$biID = $this->request->getVar('bi_id');

		if ($isMelt == '') {
			$isMelt = 0;
		}else{
			$isMelt = 1;
		}

		$batchModel = new BatchModel();
		$batchItemOutputModel = new BatchItemOutputModel();

		$batchItemModel = new BatchItemModel();

		$biData = $batchItemModel->getByID($biID);

		if ($biData == false) {
			$msg = [
                'error' => lang('Validation.data_not_found'),
            ];
            return json_encode($msg);
		}
		
		$data = [
			'bio_bi_id'=> $biID,
			'bio_oi_id'=> @$biData[0]->order_item_id,
			'bio_item_id'=> @$biData[0]->item_id,
			'bio_is_melt'=> 1,
			'bio_note'=> hexdec(uniqid()).$this->session->get('id'),
			'bio_item_sku' => @$biData[0]->sku,
			'bio_item_name' => @$biData[0]->name,
			'bio_item_quantity' => $this->request->getPost('item_quantity', FILTER_SANITIZE_STRING),
			'bio_item_weight' => $this->request->getPost('item_weight', FILTER_SANITIZE_STRING),
			'bio_insert_at' => getCurrentTime(),
			'bio_insert_admin_id' => $this->session->get('id')
		];

		$result = $batchItemOutputModel->addBatchItemOutput($data);

		if ($result) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'modal_close' => '#kt_modal_melt'
			];

			return json_encode($data);
		}
	}


	public function data(){
		// if (!$this->request->isAjax()) {
		// 	return false;
		// }

		$batchItemOutputModel = new BatchItemOutputModel();
		$batchItemOutputData = $batchItemOutputModel->getAll(['groupby' => 'batch_item_outputs.bio_bi_id', 'where' => ' AND bio_status IN ("New", "Proses") ',  'orderby' => ' bio_id ASC ']);

		$data = [];

		if (@$batchItemOutputData) {
        	foreach ($batchItemOutputData as $index => $val ) {

            $value['no'] = $index+1;
            $value['sku'] = $val->item_sku;
            $value['workflow_name'] = $val->workflow_name;
            $value['batch_code'] = $val->batch_code;
            $value['task_name'] = $val->task_name;
            $value['item_name'] = $val->item_name;
            $value['quantity'] = $val->quantity;
            $value['workflow_name'] = $val->workflow_name;
            $value['order_note'] = $val->order_note;
            $value['action'] = $btn_task.$btn_print;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
	}


}