<?php

namespace App\Controllers;

use App\Models\ItemModel;

use App\Models\WorkflowModel;
use App\Models\SizeModel;
use App\Models\GoldContentModel;
use App\Models\CategoryModel;
use Ramsey\Uuid\Uuid;
use App\Libraries\S3Library;
use App\Models\OrderItemModel;
use App\Models\GoldTypeModel;
use App\Models\ItemMaterialModel;

class Item extends BaseController
{
	public function getAll()
	{
		if($this->request->isAjax()){
			return $this->data();
		}
		
		$data = [
			'title' => lang('Menu.master.item.main'),
			'menu' => 'item',
		];

		return view('item/index', $data);
	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}
		
		$itemModel = new ItemModel();

		$itemData = $itemModel->getAll(); 

		$data = [];

		if (@$itemData) {
        	foreach ($itemData as $index => $val ) {
        		
	            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/item/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

	            $btn_edit = '&nbsp;<a href="'.base_url().'/item/edit/'.$val->id.'" class="btn btn-success btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

	            $value['no'] = $index+1;
	            $value['sku'] = $val->sku;
	            $value['name'] = $val->name;
	            $value['workflow'] = $val->workflow_name;
	            $value['category'] = $val->category_name;
	            $value['gold_content'] = $val->gc_name;
	            $value['gold_type'] = $val->gt_name;
	            $value['weight'] = $val->weight;
	            $value['size'] = $val->size_name;
	            $value['stone_type'] = $val->stone_type;
	            $value['use_wire'] = $val->use_wire;
	            $value['print_no'] = $val->print_no;
	            $value['cad_file'] = $val->cad_file;
	            $value['insert_at'] = $val->insert_at;
	            $value['action'] = $btn_edit.$btn_delete;
	            
	            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

	public function addItem()
	{
		if (!$this->request->isAjax()) {
			$sizeModel = new SizeModel();
			$workflowModel = new WorkflowModel();
			$goldContentModel = new GoldContentModel();
			$categoryModel = new CategoryModel();
			$goldTypeModel = new GoldTypeModel();

			$data = [
				'title' => lang('Menu.master.item.main'),
				'menu' => 'item',
				'data_workflow' => $workflowModel->getAll(),
				'data_size' => $sizeModel->getAll(),
				'data_category' => $categoryModel->getAll(),
				'data_gold_content' => $goldContentModel->getAll(),
				'data_gold_type' => $goldTypeModel->getAll(),
			];

			return view('item/create', $data);
		}

		$rules = [
            'sku'             => ['label' => 'SKU', 'rules' => 'required|max_length[255]'],
            'name'             => ['label' => 'Nama Barang', 'rules' => 'required|max_length[255]|' ],
            'workflow_id'             => ['label' => 'Alur Kerja', 'rules' => 'required|numeric|max_length[11]|' ],
            'category_id'             => ['label' => 'Kategori', 'rules' => 'required|numeric|max_length[11]|' ],
            'gc_id'             => ['label' => 'Kadar Emas', 'rules' => 'required|numeric|max_length[11]|' ],
            'gt_id'             => ['label' => 'Warna Emas', 'rules' => 'required|numeric|max_length[11]|' ],
            'weight'             => ['label' => 'Berat', 'rules' => 'required|numeric|max_length[11]|' ],
            // 'size_id'            => ['label' => 'Ukuran', 'rules' => 'required|numeric|max_length[11]|' ],
            'stone_type'         => ['label' => 'Jenis Batu', 'rules' => 'required|max_length[11]|' ],
            'use_wire'         => ['label' => 'Pakai Kawat', 'rules' => 'required|max_length[255]|' ],
            'print_no'         => ['label' => 'Print No', 'rules' => 'max_length[255]|' ],
            'cad_file'         => ['label' => 'Cad File', 'rules' => 'max_length[255]|' ],
        ];

        $imSource = $this->request->getPost('im_source');
		$imRequiredToCheck = $this->request->getPost('im_required_to_check');
		$imName = $this->request->getPost('im_name');
		$imQuantity = $this->request->getPost('im_quantity');

		if (isset($imSource)) {
			foreach ($imSource as $key => $value) {
				$number = $key+1;
				$rules = [
		            'im_source.'.$key => ['label' => 'Source '.$number, 'rules' => 'required|max_length[255]'],
		            'im_required_to_check.'.$key => ['label' => 'Required to check '.$number, 'rules' => 'required|max_length[255]|' ],
		            'im_name.'.$key => ['label' => 'Nama '.$number, 'rules' => 'required|max_length[255]|' ],
		            'im_quantity.'.$key => ['label' => 'Jumlah '.$number, 'rules' => 'required|numeric|max_length[11]|' ],
		        ];
			}
		}


		$fileData = $this->request->getFileMultiple('image');

		if ($fileData[0]->getClientName() == '') {
			$msg = [
                'error' =>  lang('Validation.image_required'),
            ];
            return json_encode($msg);
		}

		foreach ($fileData as $key => $value) {

			if (@$fileData[$key]->getClientName() != '') {
				$rules['image.'.$key] = ['label' => 'Image', 'rules' => 'uploaded[image.'.$key.']|mime_in[image.'.$key.',image/jpg,image/jpeg,image/gif,image/png]|max_size[image.'.$key.',4096]'];
			}
		}

		if(count($fileData)> 3){
			$msg = [
                'error' =>  lang('Validation.max_count_file'),
            ];
            return json_encode($msg);
		}

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		 	$fileData = $this->request->getFileMultiple('image');
		 	$path = '';

			$s3Library = new S3Library();

		$data = [
			'item_sku' => $this->request->getPost('sku'),
			'item_name' => $this->request->getPost('name'),
			'item_workflow_id' => $this->request->getPost('workflow_id'),
			'item_category_id' => $this->request->getPost('category_id'),
			'item_gc_id' => $this->request->getPost('gc_id'),
			'item_gt_id' => $this->request->getPost('gt_id'),
			'item_weight' => $this->request->getPost('weight'),
			'item_size_id' => $this->request->getPost('size_id'),
			'item_stone_type' => '',
			'item_use_wire' => $this->request->getPost('use_wire'),
			'item_print_no' => $this->request->getPost('print_no'),
			'item_cad_file' => $this->request->getPost('cad_file'),
            'item_insert_admin_id' => $this->session->get('id'),
            'item_active'          => 1
        ];

		$itemModel = new ItemModel();

		$itemDataBySKU = $itemModel->getBySKU($data['item_sku']);
        if ($itemDataBySKU) {
				$msg = [
	                'error' => lang('Validation.sku_already_exist'),
	            ];
	            return json_encode($msg);
        }

		$itemDataID = $itemModel->addItem($data);

		$imagePath = [];
		foreach ($fileData as $key => $value) {

            $extension = $value->guessExtension();
			$fileName = preg_replace("/[^a-zA-Z0-9-.]+/", "", date('Ymd').rand().'_'.$value->getClientName());
            $type = $value->getMimeType();
            $path = getenv('AWS_REPOSITORY_FOLDER').'/'.$itemDataID.'/'.$fileName;
            $imagePath[] = $path;
            $s3Library->putObject($path,$value);
		}

		$data = [
			'item_image_1' => json_encode($imagePath)
		];

		$itemDataUpdate = $itemModel->updateItem($data, $itemDataID);

		if (isset($imSource)) {
			$data2 = [];
			foreach ($imSource as $key => $value) {
				$data2[] = [
					'im_source'=> $imSource[$key],
					'im_required_to_check' => @$imRequiredToCheck[$key],
					'im_item_id'=> $itemDataID,
					'im_name'=> $imName[$key],
					'im_quantity'=> $imQuantity[$key],
					'im_insert_at'=> date('Y-m-d H:i:s'),
					'im_insert_admin_id'=> $this->session->get('id'),
					'im_update_at' => date('Y-m-d H:i:s'),
					'im_update_admin_id' => $this->session->get('id'),	
					'im_active' => 1,
		        ];
			}

			$ItemMaterialModel = new ItemMaterialModel();

			$ItemMaterialData = $ItemMaterialModel->addItemMaterialBulk($data2);
		}

		
		if ($itemDataID) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect' => 'item'
			];

			return json_encode($data);
		}

	}

	public function updateItem($id)
	{
		$itemModel = new ItemModel();

		$itemData = $itemModel->getByID($id);

		if (!$this->request->isAjax()) {

			$sizeModel = new SizeModel();
			$workflowModel = new WorkflowModel();
			$goldContentModel = new GoldContentModel();
			$categoryModel = new CategoryModel();
			$goldTypeModel = new GoldTypeModel();

			$ItemMaterialModel = new ItemMaterialModel();
			$param['where'] = ' AND im_item_id="'.$id.'" AND im_active=1 ';
			$ItemMaterialData = $ItemMaterialModel->getAll($param); 

			$data = [
				'title' => lang('Menu.master.item.main'),
				'data' => (isset($itemData))? $itemData : [],
				'menu' => 'item',
				'itemID' => $id,
				'itemMaterialData' => $ItemMaterialData,
				'data_workflow' => $workflowModel->getAll(),
				'data_size' => $sizeModel->getAll(),
				'data_category' => $categoryModel->getAll(),
				'data_gold_content' => $goldContentModel->getAll(),
				'data_gold_type' => $goldTypeModel->getAll(),
			];

			return view('item/edit', $data);
		}

		if (!$this->validate([
            //-- 'sku'             => ['label' => 'SKU', 'rules' => 'required|max_length[255]'],
            // 'name'             => ['label' => 'Nama', 'rules' => 'required|max_length[255]|' ],
            'workflow_id'             => ['label' => 'Alur Kerja', 'rules' => 'required|numeric|max_length[11]|' ],
            // 'category_id'             => ['label' => 'Kategori', 'rules' => 'required|numeric|max_length[11]|' ],
            // 'gc_id'             => ['label' => 'Kadar Emas', 'rules' => 'required|numeric|max_length[11]|' ],
            // 'gt_id'             => ['label' => 'Warna Emas', 'rules' => 'required|numeric|max_length[11]|' ],
            // 'weight'             => ['label' => 'Berat', 'rules' => 'required|numeric|max_length[11]|' ],
            //-- 'size_id'            => ['label' => 'Ukuran', 'rules' => 'required|numeric|max_length[11]|' ],
            // 'stone_type'         => ['label' => 'Jenis Batu', 'rules' => 'required|max_length[11]|' ],
            // 'use_wire'         => ['label' => 'Pakai Kawat', 'rules' => 'required|max_length[255]|' ],
            // 'print_no'         => ['label' => 'Print No', 'rules' => 'max_length[255]|' ],
            // 'cad_file'         => ['label' => 'Cad File', 'rules' => 'max_length[255]|' ],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$imSource = $this->request->getPost('im_source');

		$imRequiredToCheck = $this->request->getPost('im_required_to_check');
		$imName = $this->request->getPost('im_name');
		$imQuantity = $this->request->getPost('im_quantity');

		if (isset($imSource)) {
			foreach ($imSource as $key => $value) {
				$number = $key+1;
				$rules = [
		            'im_source.'.$key => ['label' => 'Source '.$number, 'rules' => 'required|max_length[255]'],
		            'im_required_to_check.'.$key => ['label' => 'Required to check '.$number, 'rules' => 'required|max_length[255]|' ],
		            'im_name.'.$key => ['label' => 'Nama '.$number, 'rules' => 'required|max_length[255]|' ],
		            'im_quantity.'.$key => ['label' => 'Jumlah '.$number, 'rules' => 'required|numeric|max_length[11]|' ],
		        ];
			}
		}

		 	$fileData = $this->request->getFileMultiple('image');
		 	$path = '';

			$s3Library = new S3Library();

			$imagePath = $this->request->getPost('image_old');

			if (@$fileData) {
				foreach ($fileData as $key => $value) {
					if ($value->getClientName() != '') {
						$fileName = preg_replace("/[^a-zA-Z0-9]+/", "", $value->getClientName());
			            $extension = $value->guessExtension();
			            $type = $value->getMimeType();
			            $path = getenv('AWS_REPOSITORY_FOLDER').'/'.$this->session->get('id').'/'.time().'_'.$fileName;
			            $imagePath[] = $path;
			            $s3Library->putObject($path,$value);
					}
				}
			}

		$data = [
			//-- 'item_sku' => $this->request->getPost('sku'),
			// 'item_name' => $this->request->getPost('name'),
			'item_workflow_id' => $this->request->getPost('workflow_id'),
			// 'item_category_id' => $this->request->getPost('category_id'),
			// 'item_gc_id' => $this->request->getPost('gc_id'),
			// 'item_gt_id' => $this->request->getPost('gt_id'),
			// 'item_weight' => $this->request->getPost('weight'),
			// 'item_size_id' => $this->request->getPost('size_id'),
			// 'item_stone_type' => $this->request->getPost('stone_type'),
			// 'item_use_wire' => $this->request->getPost('use_wire'),
			// 'item_image_1' => json_encode($imagePath),
			// 'item_print_no' => $this->request->getPost('print_no'),
			// 'item_cad_file' => $this->request->getPost('cad_file'),
            'item_update_admin_id' => $this->session->get('id'),
            'item_active'          => 1
        ];

		// if ($itemData->sku != $data['item_sku']) {
		// 	$itemDataBySKU = $itemModel->getBySKU($data['item_sku']);
	 //        if ($itemDataBySKU) {
		// 			$msg = [
		//                 'error' => lang('Validation.sku_already_exist'),
		//             ];
		//             return json_encode($msg);
	 //        }
		// }

		$itemData = $itemModel->updateItem($data, $id); 



		if (isset($imSource)) {
			$data2 = [];
			foreach ($imSource as $key => $value) {
				if($value != ''){
					$data2[] = [
						'im_source'=> $imSource[$key],
						'im_required_to_check' => (@$imRequiredToCheck[$key])? 1 : 0,
						'im_item_id'=> $id,
						'im_name'=> $imName[$key],
						'im_quantity'=> $imQuantity[$key],
						'im_insert_at'=> date('Y-m-d H:i:s'),
						'im_insert_admin_id'=> $this->session->get('id'),
						'im_update_at' => date('Y-m-d H:i:s'),
						'im_update_admin_id' => $this->session->get('id'),	
						'im_active' => 1,
			        ];
				}
			}

			if (!empty($data2)) {
				$ItemMaterialModel = new ItemMaterialModel();

				$ItemMaterialDelete = $ItemMaterialModel->deleteItemMaterialByItem($id, $this->session->get('id'));

				$ItemMaterialData = $ItemMaterialModel->addItemMaterialBulk($data2);
			}
		}

		if ($itemData) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'item'
			];

			return json_encode($data);
		}
	}

	public function deleteItem($id)
	{
		$orderItemModel = new OrderItemModel();
		if(count($orderItemModel->getAll(["where" => ' AND oi_item_id="'.$id.'" AND item_active=1 AND oi_active=1 AND order_active=1 '])) > 0 ){
			$data = [
				'error'=> lang('Validation.item_already_use'),
			];
			return json_encode($data);
		}

		$itemModel = new ItemModel();

		$itemData = $itemModel->deleteItem($id, $this->session->get('id')); 

		if ($itemData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}

}