<?php
namespace App\Controllers;

use App\Models\OrderModel;
use App\Models\OrderItemModel;
use App\Models\BatchItemModel;
use App\Models\CustomerModel;
use App\Models\WorkflowModel;
use App\Models\SizeModel;
use App\Models\GoldContentModel;
use App\Models\GoldTypeModel;
use App\Models\TreeModel;
use App\Models\CategoryModel;
use App\Models\TaskModel;
use App\Models\ItemModel;
use Ramsey\Uuid\Uuid;
use App\Libraries\S3Library;

class Report extends BaseController
{
	public function order()
	{
        if($this->request->isAjax()){
            return $this->dataOrder();
        }

        if ($this->request->getGet('export-excel') == true) {
            return $this->exportOrder();
        }

        $workflowModel = new WorkflowModel();
        $customerModel = new CustomerModel();
        $sizeModel = new SizeModel();
        $goldContentModel = new GoldContentModel();
        $goldTypeModel = new GoldTypeModel();
        $categoryModel = new CategoryModel();
        $taskModel = new TaskModel();
        $itemModel = new ItemModel();
        $treeModel = new treeModel();

		$data = [
            'data_workflow'=> $workflowModel->getAll(),
            'data_customer'=> $customerModel->getAll(),
            'data_size'=> $sizeModel->getAll(),
            'data_gold_content'=> $goldContentModel->getAll(),
            'data_gold_type'=> $goldTypeModel->getAll(),
            'data_category'=> $categoryModel->getAll(),
            'data_task'=> $taskModel->getAll(),
            'data_item'=> $itemModel->getAll(),
            'data_tree'=> $treeModel->getAll(),
			'title' => lang('Menu.report.order.main'),
			'menu' => 'order-report',
		];

		return view('report/order', $data);
	}

	public function dataOrder()
	{
		if (!$this->request->isAjax()) {
			return false;
		}

        $fromDate = $this->request->getGet('from_date');
        $toDate = $this->request->getGet('to_date');
        $workflowId = $this->request->getGet('workflow_id');
        $customerId = $this->request->getGet('customer_id');
        $sizeId = $this->request->getGet('size_id');
        $gcId = $this->request->getGet('gc_id');
        $gtId = $this->request->getGet('gt_id');
        $categoryId = $this->request->getGet('category_id');
        $taskId = $this->request->getGet('task_id');
        $itemId = $this->request->getGet('item_id');
        $treeId = $this->request->getGet('tree_id');
        $status = $this->request->getGet('status');
        $orderNote = $this->request->getGet('note');

        $queryWhere = '';

        if ($fromDate != '') {
            $queryWhere .= ' AND order_date >="'.$fromDate.'" ';
        }

        if ($toDate != '') {
            $queryWhere .= ' AND order_date <= "'.$toDate.'" ';
        }

        if ($workflowId != '') {
            $queryWhere .= ' AND item_workflow_id = "'.$workflowId.'" ';
        }

        if ($customerId != '') {
            $queryWhere .= ' AND order_customer_id = "'.$customerId.'" ';
        }

        if ($sizeId != '') {
            $queryWhere .= ' AND item_size_id = "'.$sizeId.'" ';
        }

        if ($gcId != '') {
            $queryWhere .= ' AND item_gc_id = "'.$gcId.'" ';
        }

        if ($gtId != '') {
            $queryWhere .= ' AND item_gt_id = "'.$gtId.'" ';
        }

        if ($categoryId != '') {
            $queryWhere .= ' AND item_category_id = "'.$categoryId.'" ';
        }

        if ($taskId != '') {
            $queryWhere .= ' AND wt_task_id = "'.$taskId.'" ';
        }

        if ($itemId != '') {
            $queryWhere .= ' AND items.item_id = "'.$itemId.'" ';
        }

        if ($status != '') {
            $queryWhere .= ' AND orders.order_status = "'.$status.'" ';
        }

        if ($orderNote != '') {
            $queryWhere .= ' AND orders.order_note = "'.$orderNote.'" ';
        }

		$orderItemModel = new OrderItemModel();

		$orderItemData = $orderItemModel->getAll(['where'=> $queryWhere]);

		$data = [];

		if (@$orderItemData) {
        	foreach ($orderItemData as $index => $val ) {

            $value['no'] = ($index+1);
            $value['date'] = $val->date;
            $value['due_date'] = $val->due_date;
            $value['start_date'] = $val->start_date;
            $value['name'] = $val->name;
            $value['note'] = '<a href="'.base_url('batch-item-output?order_item='.$val->id).'">'.$val->order_note.'</a>';
            $value['customer_name'] = $val->customer_name;
            $value['customer_phone'] = $val->customer_phone;
            $value['customer_address'] = $val->customer_address;
            $value['quantity'] = $val->quantity;
            $value['completed_quantity'] = $val->completed_quantity;
            $value['status'] = $val->order_item_status;
            $value['category'] = $val->category_name;
            $value['workflow'] = $val->workflow_name;
            $value['gold_content'] = $val->gc_name;
            $value['gold_type'] = $val->gt_name;
            $value['weight'] = $val->weight;
            $value['size'] = $val->size_name;
            $value['use_wire'] = $val->use_wire;
            $value['stone_type'] = $val->stone_type;
            $value['print_no'] = $val->print_no;

            $value['user_update_id'] = $val->user_update;
            $value['user_insert_id'] = $val->user_insert;
            $value['user_delete_id'] = $val->user_delete;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

    public function exportOrder(){
        $fromDate = $this->request->getGet('from_date');
        $toDate = $this->request->getGet('to_date');
        $workflowId = $this->request->getGet('workflow_id');
        $customerId = $this->request->getGet('customer_id');
        $sizeId = $this->request->getGet('size_id');
        $gcId = $this->request->getGet('gc_id');
        $gtId = $this->request->getGet('gt_id');
        $categoryId = $this->request->getGet('category_id');
        $taskId = $this->request->getGet('task_id');
        $itemId = $this->request->getGet('item_id');
        $treeId = $this->request->getGet('tree_id');
        $status = $this->request->getGet('status');
        $orderNote = $this->request->getGet('note');

        $queryWhere = '';

        if ($fromDate != '') {
            $queryWhere .= ' AND order_date >="'.$fromDate.'" ';
        }

        if ($toDate != '') {
            $queryWhere .= ' AND order_date <= "'.$toDate.'" ';
        }

        if ($workflowId != '') {
            $queryWhere .= ' AND item_workflow_id = "'.$workflowId.'" ';
        }

        if ($customerId != '') {
            $queryWhere .= ' AND order_customer_id = "'.$customerId.'" ';
        }

        if ($sizeId != '') {
            $queryWhere .= ' AND item_size_id = "'.$sizeId.'" ';
        }

        if ($gcId != '') {
            $queryWhere .= ' AND item_gc_id = "'.$gcId.'" ';
        }

        if ($gtId != '') {
            $queryWhere .= ' AND item_gt_id = "'.$gtId.'" ';
        }

        if ($categoryId != '') {
            $queryWhere .= ' AND item_category_id = "'.$categoryId.'" ';
        }

        if ($taskId != '') {
            $queryWhere .= ' AND wt_task_id = "'.$taskId.'" ';
        }

        if ($itemId != '') {
            $queryWhere .= ' AND items.item_id = "'.$itemId.'" ';
        }

        if ($status != '') {
            $queryWhere .= ' AND orders.order_status = "'.$status.'" ';
        }

        if ($orderNote != '') {
            $queryWhere .= ' AND orders.order_note = "'.$orderNote.'" ';
        }

        $orderItemModel = new OrderItemModel();

        $orderItemData = $orderItemModel->getAll(['where'=> $queryWhere]);

        $data = [
            'title' => lang('Menu.report.order.main'),
            'data' => $orderItemData,
        ];

        return view('report/export-order', $data);
    }


    public function batch()
	{
        if($this->request->isAjax()){
            return $this->dataBatch();
        }

        if ($this->request->getGet('export-excel') == true) {
            return $this->exportBatch();
        }

        $workflowModel = new WorkflowModel();
        $customerModel = new CustomerModel();
        $sizeModel = new SizeModel();
        $goldContentModel = new GoldContentModel();
        $goldTypeModel = new GoldTypeModel();
        $categoryModel = new CategoryModel();
        $taskModel = new TaskModel();
        $itemModel = new ItemModel();
        $treeModel = new treeModel();

        $data = [
            'data_workflow'=> $workflowModel->getAll(),
            'data_customer'=> $customerModel->getAll(),
            'data_size'=> $sizeModel->getAll(),
            'data_gold_content'=> $goldContentModel->getAll(),
            'data_gold_type'=> $goldTypeModel->getAll(),
            'data_category'=> $categoryModel->getAll(),
            'data_task'=> $taskModel->getAll(),
            'data_item'=> $itemModel->getAll(),
            'data_tree'=> $treeModel->getAll(),
			'title' => lang('Menu.report.batch.main'),
			'menu' => 'batch-report',
		];

		return view('report/batch', $data);
	}

	public function dataBatch()
	{
		if (!$this->request->isAjax()) {
			return false;
		}


        $fromDate = $this->request->getGet('from_date');
        $toDate = $this->request->getGet('to_date');
        $workflowId = $this->request->getGet('workflow_id');
        $customerId = $this->request->getGet('customer_id');
        $sizeId = $this->request->getGet('size_id');
        $gcId = $this->request->getGet('gc_id');
        $gtId = $this->request->getGet('gt_id');
        $categoryId = $this->request->getGet('category_id');
        $taskId = $this->request->getGet('task_id');
        $itemId = $this->request->getGet('item_id');
        $treeId = $this->request->getGet('tree_id');
        $status = $this->request->getGet('status');
        $batchCode = $this->request->getGet('batch_code');
        $orderNote = $this->request->getGet('note');

        $queryWhere = '';

        if ($fromDate != '') {
            $queryWhere .= ' AND batch_start_date >="'.$fromDate.'" ';
        }

        if ($toDate != '') {
            $queryWhere .= ' AND batch_start_date <= "'.$toDate.'" ';
        }

        if ($workflowId != '') {
            $queryWhere .= ' AND item_workflow_id = "'.$workflowId.'" ';
        }

        if ($customerId != '') {
            $queryWhere .= ' AND order_customer_id = "'.$customerId.'" ';
        }

        if ($sizeId != '') {
            $queryWhere .= ' AND item_size_id = "'.$sizeId.'" ';
        }

        if ($gcId != '') {
            $queryWhere .= ' AND item_gc_id = "'.$gcId.'" ';
        }

        if ($gtId != '') {
            $queryWhere .= ' AND item_gt_id = "'.$gtId.'" ';
        }

        if ($categoryId != '') {
            $queryWhere .= ' AND item_category_id = "'.$categoryId.'" ';
        }

        if ($taskId != '') {
            $queryWhere .= ' AND wt_task_id = "'.$taskId.'" ';
        }

        if ($itemId != '') {
            $queryWhere .= ' AND items.item_id = "'.$itemId.'" ';
        }

        if ($status != '') {
            $queryWhere .= ' AND batches.batch_status = "'.$status.'" ';
        }

        if ($batchCode != '') {
            $queryWhere .= ' AND batches.batch_code = "'.$batchCode.'" ';
        }

        if ($orderNote != '') {
            $queryWhere .= ' AND orders.order_note = "'.$orderNote.'" ';
        }
		
		$batchItemModel = new BatchItemModel();

		$batchItemData = $batchItemModel->getAll(['where'=> $queryWhere]);

		$data = [];

		if (@$batchItemData) {
        	foreach ($batchItemData as $index => $val ) {

            $value['no'] = ($index+1);
            $value['start_date'] = $val->start_date;
            $value['note'] = $val->order_note;
            $value['batch_code'] = $val->batch_code;
            $value['name'] = $val->name;
            $value['customer_name'] = $val->customer_name;
            $value['customer_phone'] = $val->customer_phone;
            $value['customer_address'] = $val->customer_address;
            $value['category'] = $val->category_name;
            $value['gold_content'] = $val->gc_name;
            $value['quantity'] = $val->quantity;
            $value['completed_quantity'] = $val->completed_quantity;
            $value['status'] = $val->batch_status;
            $value['weight'] = $val->weight;
            $value['size'] = $val->size_name;
            $value['use_wire'] = $val->use_wire;
            $value['stone_type'] = $val->stone_type;
            $value['print_no'] = $val->print_no;
            $value['user_update_id'] = $val->user_update;
            $value['user_insert_id'] = $val->user_insert;
            $value['user_delete_id'] = $val->user_delete;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

    public function exportBatch(){
        $fromDate = $this->request->getGet('from_date');
        $toDate = $this->request->getGet('to_date');
        $workflowId = $this->request->getGet('workflow_id');
        $customerId = $this->request->getGet('customer_id');
        $sizeId = $this->request->getGet('size_id');
        $gcId = $this->request->getGet('gc_id');
        $gtId = $this->request->getGet('gt_id');
        $categoryId = $this->request->getGet('category_id');
        $taskId = $this->request->getGet('task_id');
        $itemId = $this->request->getGet('item_id');
        $treeId = $this->request->getGet('tree_id');
        $status = $this->request->getGet('status');
        $batchCode = $this->request->getGet('batch_code');
        $orderNote = $this->request->getGet('note');

        $queryWhere = '';

        if ($fromDate != '') {
            $queryWhere .= ' AND batch_start_date >="'.$fromDate.'" ';
        }

        if ($toDate != '') {
            $queryWhere .= ' AND batch_start_date <= "'.$toDate.'" ';
        }

        if ($workflowId != '') {
            $queryWhere .= ' AND item_workflow_id = "'.$workflowId.'" ';
        }

        if ($customerId != '') {
            $queryWhere .= ' AND order_customer_id = "'.$customerId.'" ';
        }

        if ($sizeId != '') {
            $queryWhere .= ' AND item_size_id = "'.$sizeId.'" ';
        }

        if ($gcId != '') {
            $queryWhere .= ' AND item_gc_id = "'.$gcId.'" ';
        }

        if ($gtId != '') {
            $queryWhere .= ' AND item_gt_id = "'.$gtId.'" ';
        }

        if ($categoryId != '') {
            $queryWhere .= ' AND item_category_id = "'.$categoryId.'" ';
        }

        if ($taskId != '') {
            $queryWhere .= ' AND wt_task_id = "'.$taskId.'" ';
        }

        if ($itemId != '') {
            $queryWhere .= ' AND items.item_id = "'.$itemId.'" ';
        }

        if ($status != '') {
            $queryWhere .= ' AND batches.batch_status = "'.$status.'" ';
        }

        if ($batchCode != '') {
            $queryWhere .= ' AND batches.batch_code = "'.$batchCode.'" ';
        }

        if ($orderNote != '') {
            $queryWhere .= ' AND orders.order_note = "'.$orderNote.'" ';
        }

        $batchItemModel = new BatchItemModel();

        $batchItemData = $batchItemModel->getAll(['where'=> $queryWhere]);

        $data = [
            'title' => lang('Menu.report.order.main'),
            'data' => $batchItemData,
        ];

        return view('report/export-batch', $data);
    }

}