<?php namespace App\Models;

class OrderItemModel extends BaseModel
{
    public function __construct(){
        parent::__construct();
    }

    public function getAll($param=array()){
        if (!empty($param['select']))
            $query = 'SELECT '.$param["select"];
        else
            $query = 'SELECT order_items.oi_id as "id", items.item_sku as "sku", items.item_id, items.item_stone_type, workflows.workflow_id, order_items.oi_item_name as "name", categories.category_name, categories.category_id, gold_contents.gc_name,gold_types.gt_id, gold_types.gt_name, gold_contents.gc_id, gold_contents.gc_multiplier as "multiplier", order_items.oi_weight as "weight", sizes.size_name, sizes.size_id, items.item_stone_type as "stone_type", order_items.oi_quantity as "quantity", order_items.oi_completed_quantity as "completed_quantity", orders.order_note, items.item_use_wire as "use_wire", items.item_print_no as "print_no", order_items.oi_start_date as "start_date", customers.customer_name, customers.customer_address, customers.customer_phone, orders.order_status as "order_item_status", order_items.oi_start_date, order_items.oi_due_date as "due_date", workflows.workflow_name, items.item_image_1 as "image", orders.order_date as "date", ins.admin_fullname as "user_insert", upd.admin_fullname as "user_update", del.admin_fullname as "user_delete" ';

        $query .= ' FROM order_items 
            INNER JOIN orders ON orders.order_id=order_items.oi_order_id
            INNER JOIN items ON items.item_id=order_items.oi_item_id 
            INNER JOIN customers ON orders.order_customer_id=customers.customer_id  
            INNER JOIN categories ON items.item_category_id=categories.category_id 
            LEFT JOIN sizes ON order_items.oi_size_id=sizes.size_id
            LEFT JOIN gold_contents ON order_items.oi_gc_id=gold_contents.gc_id 
            LEFT JOIN gold_types ON items.item_gt_id=gold_types.gt_id 
            LEFT JOIN workflows ON workflows.workflow_id=items.item_workflow_id 
            LEFT JOIN workflow_tasks ON workflow_tasks.wt_workflow_id=workflows.workflow_id 
            LEFT JOIN tasks ON tasks.task_id=workflow_tasks.wt_task_id 
            LEFT JOIN admins ins ON ins.admin_id=order_items.oi_insert_admin_id
            LEFT JOIN admins upd ON upd.admin_id=order_items.oi_update_admin_id
            LEFT JOIN admins del ON del.admin_id=order_items.oi_delete_admin_id
            WHERE 1';

        if(!empty($param['where']))
            $query .= ' '.$param['where'];
        else
            $query .= ' AND order_active=1 AND oi_active=1 ';

        if(!empty($param['groupby']))
            $query .= ' GROUP BY '.$param['groupby'];
        else
            $query .= ' GROUP BY oi_id ';

        if(!empty($param['orderBy']))
            $query .= ' ORDER BY '.$param['orderby'];
        else
            $query .= ' ORDER BY 1 DESC';

        if(!empty($param['limit']))
            $query .= ' LIMIT '.$param['limit'];
        else
            $query .= '';

        if(!empty($param['resultType'])) 
            return $this->db->query($query);
        else
            return $this->db->query($query)->getResult();
    }

    public function getByID($oiID){
        $param['where'] = ' AND oi_id='.$oiID.' AND oi_active=1 ';

        $result = $this->getAll($param);

        return $result[0];
    }

    public function getByOrderID($orderID){
        $param['where'] = ' AND oi_order_id='.$orderID.' AND oi_active=1 ';

        $result = $this->getAll($param);

        return $result;
    }

    public function getByOrderIDStatus($orderID, $status){
        $param['where'] = ' AND orders.order_id='.$orderID.' AND order_items.oi_status="'.$status.'" AND order_active=1 ';

        $result = $this->getAll($param);

        return $result;
    }

    public function addOrderItem($data){
        $oi = $this->db->table('order_items');
        $oi->insert($data);
        $result = $this->db->insertID();

        return $result;
    }

    public function addOrderItemBulk($data){
        $oi = $this->db->table('order_items');
        $oi->insertBatch($data);

        return $data;
    }

    public function updateOrderItem($data, $oiID){

        $this->db->table('order_items')->where('oi_id',$oiID)->update($data);
        $param = array(
            'where' => ' AND oi_id="'.$oiID.'" ',
        );
        $result = $this->getAll($param);
        if ($result)
            $result = $result[0];

        return $result;
        
    }

    public function deleteOrder($oiID, $deleteAdminID ){
        $data = [
            'oi_delete_at'       => getCurrentTime(),
            'oi_delete_admin_id' => $deleteAdminID,
            'oi_active'          => 0
        ];

        $this->db->table('order_items')->where('oi_id',$oiID)->update($data);
        $param = array(
            'where' => ' AND oi_id="'.$oiID.'" ',
        );
        $result = $this->getAll($param);
        if ($result)
            $result = $result[0];

        return $result;
        
    }

    public function deleteOrderItemByOrderID($orderID, $deleteAdminID ){
        $data = [
            'oi_delete_at'       => getCurrentTime(),
            'oi_delete_admin_id' => $deleteAdminID,
            'oi_active'          => 0,
            'oi_status'         => 'Deleted',
        ];

        $this->db->table('order_items')->where('oi_order_id',$orderID)->update($data);
        $param = array(
            'where' => ' AND oi_order_id="'.$orderID.'" ',
        );
        $result = $this->getAll($param);
        if ($result)
            $result = $result[0];

        return $result;
    }
}