<?php 
namespace App\Validation;

class CustomRules{

    public function gender(string $str, string &$error = null): bool
    {
        if (!in_array($str, ['male', 'female']))
        {
            $error = lang('ErrorMessages.invalidGenderMaleFemale');
            return FALSE;
        }

        return TRUE;
    }

    public function bloodType($str, string &$error = null): bool{
        if(!empty($str) && !in_array(strtoupper($str), ['A', 'B', 'AB', 'O'])){
            $error = lang('ErrorMessages.invalidBloodTypeABABO');
            return FALSE;
        }

        return TRUE;
    }

    public function maritalStatus($str, string &$error = null): bool{
        if(!empty($str) && !in_array(strtolower($str), ['single', 'married'])){
            $error = lang('ErrorMessages.invalidMaritalStatusSingleMarried');
            return FALSE;
        }

        return TRUE;
    }

    public function dateFormat($date, string &$error = null): bool{
        $d = \DateTime::createFromFormat('Y-m-d', $date);
        if(!($d && ($d->format('Y-m-d') === $date))){
            $error = lang('ErrorMessages.invalidDateFormat');
            return FALSE;
        }

        return TRUE;
    }

    public function timeFormat($time, string &$error = null): bool{
        $d = \DateTime::createFromFormat('H:i:s', $time);
        if(!($d && ($d->format('H:i:s') === $time))){
            $error = lang('ErrorMessages.invalidTimeFormat');
            return FALSE;
        }

        return TRUE;
    }

    public function validEmail($str, string &$error = null): bool{
        if (!empty($str) && !filter_var($str, FILTER_VALIDATE_EMAIL)) {
            $error = lang('ErrorMessages.invalidEmailFormat');
            return FALSE;
        }

        return TRUE;
    }
    

    public function validPhone($phoneNumber, string &$error = null): bool{
        if (!empty($phoneNumber) && \substr($phoneNumber, 0, 1)!="0" && \substr($phoneNumber, 0, 2)!="62" && \substr($phoneNumber, 0, 3)!="+62") {
            $error = "Invalid phone number, it must be prefix with 0 OR +62 OR 62";
            return FALSE;
        }

        return TRUE;
    }




}